"""
--------------------------------------------------------------------------------
Name:        get_cpt matplotlib colormap utility
Purpose:     an easy way to fetch .cpt colormap files, based on pycpt

Created:     2020.03
Copyright:   (c) Dimitrios Bouziotas (bouziot)
Licence:     GGNU General Public License v3 (GPL-3)
-You may freely copy, distribute and modify the software, in accordance with the provided license.
-You may not sublicense or hold the original author liable. This software comes with no warranty at all.
-Active contributions, forks and redevelopments are welcome.
-If you would like to include this software in your work, please reference it using the zenodo or github link. Please
also reference the original pycpt package (https://github.com/j08lue/pycpt)
--------------------------------------------------------------------------------
"""

__version__ = '0.1.0'
__copyright__ = """(c) 2020 Dimitrios Bouziotas"""
__license__ = "GGNU General Public License v3 (GPL-3)"

import os
import numpy as np
import matplotlib.colors as mcolors
import matplotlib.pyplot as plt

from urllib.request import urlopen

basedir = os.path.join(os.getcwd(),'cpt') # base path from where cpt files are fetched

def get_cmap(cpt_path, name=None, method='cdict', ret_cmap_type='LinearSegmented', N=None):
    """Get the cpt colormap as a LinearSegmented colormap. Utilizes the gmtColormap_openfile parser.
    Parameters
    ----------
    cpt_path : str, with one of the following options:
        - the full dir path to a .cpt file
        - the filename of a .cpt file that is in the local repo (check get_cpt.basedir)
        - a url.

    name : str, optional
        colormap name
        if not provided, the name will be derived automatically using _getname()

    method: str, optional
        Choose between 'cdict' and 'list'. The first one fetches all info from the .cpt file. The latter
        allows you to control the number of colors to sample from. Check gmtColormap_openfile for more info.

    N: int, optional
        the number of colors in the colormap to be returned. Can define the granularity of the returned colormap.
        Only useful when method='list'
    """
    # first get name
    if name is None:
        name = _getname(cpt_path)
    # case URL
    if 'http://' in cpt_path or 'https://' in cpt_path:
        with urlopen(cpt_path) as f:
            return gmtColormap_openfile(f, name=name, method=method, N=N, ret_cmap_type=ret_cmap_type)
    # case FULLPATH OR NAME
    else:
        with open(cpt_path) as f:
            return gmtColormap_openfile(f, name=name, method=method, N=N, ret_cmap_type=ret_cmap_type)

def get_listed_cmap(cpt_path, name=None, N=None):
    """Get the cpt colormap as a ListedColormap. Utilizes the gmtColormap_openfile parser.
    Parameters
    ----------
    cpt_path : str, with one of the following options:
        - the full dir path to a .cpt file
        - the filename of a .cpt file that is in the local repo (check get_cpt.basedir)
        - a url

    name : str, optional
        colormap name
        if not provided, the name will be derived automatically using _getname()

    N: int, optional
        the number of colors in the colormap to be returned. Leave None to derive the colors from the .cpt file.
        If you use a number less than the colors included in that file, a subset of colors will be returned.
    """
    # first get name
    if name is None:
        name = _getname(cpt_path)
    # case URL
    if 'http://' in cpt_path or 'https://' in cpt_path:
        with urlopen(cpt_path) as f:
            return gmtColormap_openfile(f, name=name, method='list', N=N, ret_cmap_type='Listed')
    # case FULLPATH OR NAME
    else:
        with open(cpt_path) as f:
            return gmtColormap_openfile(f, name=name, method='list', N=N, ret_cmap_type='Listed')

def _getname(cpt_path):
    """Internal function, fetches the name from a cpt filepath or url.
    Templates:
    'my.mby.cpt' -> 'my_mby' # NAME
    r'D:\matplotlib colormaps - cpt-city\cpt\mby.cpt' -> 'mby'  # FULLPATH
    'http://soliton.vm.bytemark.co.uk/pub/cpt-city/cmocean/haline.cpt'  -> 'haline' # URL
    """
    if 'http://' in cpt_path or 'https://' in cpt_path:  # CASE URL
        return '_'.join(cpt_path.split(r'/')[-1].split('.')[:-1])
    else:
        if os.path.exists(cpt_path):
            return cpt_path
        else:
            raise FileNotFoundError("CPT file {} not found".format(cpt_path))

def gmtColormap_openfile(cptf, name=None, method='cdict', N=None, ret_cmap_type='LinearSegmented'):
    """Read a GMT color map from an OPEN cpt file
    Edited by: bouziot, 2020.03

    Parameters
    ----------
    cptf : str, open file or url handle
        path to .cpt file

    name : str, optional
        name for color map
        if not provided, the file name will be used

    method : str, suggests the method to use.
    If method = 'cdict', generates the LinearSegmentedColormap using a color dictionary (cdict), disregarding any value in N.
    If method = 'list', generates the LinearSegmentedColor using the .from_list() method, passing a list of (value, (r,g,b)) tuples obtained from the cpt file. This allows the selection of colormap resolution by the user, using the N parameter

    N : int, the number of colors in the colormap. Only useful when method='list'.

    ret_cmap_type: str, the type of matplotlib cmap object to be returned. Accepts either 'LinearSegmented', which returns a matplotlib.colors.LinearSegmentedColormap, or 'Listed', which returns a ListedColormap
    In case 'Listed' is selected, the method argument from the user is ignored and method is set to 'list' ('Linear' doesn't work with 'cdict').
    N is then passed to matplotlib.colors.ListedColormap().
    - If N is set to None: all colors of the cpt file will be returned as a list.
    - In case of a user-defined N, colors will be truncated or extended by repetition (see matplotlib.colors.ListedColormap).

    Returns
    -------
    a matplotlib colormap object (matplotlib.colors.LinearSegmentedColormap or matplotlib.colors.ListedColormap)

    Credits
    -------
    This function originally appears in pycpt, extensive edits from bouziot, 2020.03
    Original work in: https://github.com/j08lue/pycpt
    LOG OF EDITS (2020.03):
        - Fixed bug when parsing non-split '#' lines in .cpt files
        - Fixed bug - not identifying the colorModel '#' line correctly
        - Fixed binary comparison performance (introduced in python 3)
        - Added functionality to return ListedColormaps and cmaps with custom colors (method, ret_cmap_type args)
        - Added global name handling externally (_getname() func)
    """
    methodnames = ['cdict', 'list'] # accepted method arguments
    ret_cmap_types = ['LinearSegmented', 'Listed', 'raw'] # accepted return matplotlib colormap types

    # generate cmap name
    if name is None:
        name = _getname(cptf.name)
    #    name = '_'.join(os.path.basename(cptf.name).split('.')[:-1])

    # process file
    x = []
    r = []
    g = []
    b = []
    lastls = None
    for l in cptf.readlines():
        ls = l.split()

        # skip empty lines
        if not ls:
            continue

        # parse header info
        # this leads to mistakes in some files...
        # if ls[0] in ["#", b"#"]:  # '#' is not always separated from other letters in some cases...
        #    if ls[-1] in ["HSV", b"HSV"]:
        #        colorModel = "HSV"
        #    else:
        #        colorModel = "RGB"
        #    continue

        # byte comparison is not feasible in python 3
        if (isinstance(l, bytes) and l.decode('utf-8')[0] in ["#", b"#"]) or (isinstance(l, str) and l[0] in ["#", b"#"]):
            if ls[-1] in ["HSV", b"HSV"]:
                colorModel = "HSV"
                continue
            elif ls[-1] in ["RGB", b"RGB"]:
                colorModel = "RGB"
                continue
            else: # case rogue comment, ignore
                continue


        # skip BFN info
        if ls[0] in ["B", b"B", "F", b"F", "N", b"N"]:
            continue

        # parse color vectors
        x.append(float(ls[0]))
        r.append(float(ls[1]))
        g.append(float(ls[2]))
        b.append(float(ls[3]))

        # save last row
        lastls = ls

    # check if last endrow has the same color, if not, append
    if not ((float(lastls[5]) == r[-1]) and (float(lastls[6]) == g[-1]) and (float(lastls[7]) == b[-1])):
        x.append(float(lastls[4]))
        r.append(float(lastls[5]))
        g.append(float(lastls[6]))
        b.append(float(lastls[7]))

    x = np.array(x)
    r = np.array(r)
    g = np.array(g)
    b = np.array(b)

    if colorModel == "HSV":
        for i in range(r.shape[0]):
            # convert HSV to RGB
            rr,gg,bb = colorsys.hsv_to_rgb(r[i]/360., g[i], b[i])
            r[i] = rr ; g[i] = gg ; b[i] = bb
    elif colorModel == "RGB":
        r /= 255.
        g /= 255.
        b /= 255.

    red = []
    blue = []
    green = []
    xNorm = (x - x[0])/(x[-1] - x[0])

    # return colormap
    if method == 'cdict' and ret_cmap_type == 'LinearSegmented':
        # generate cdict
        for i in range(len(x)):
            red.append([xNorm[i],r[i],r[i]])
            green.append([xNorm[i],g[i],g[i]])
            blue.append([xNorm[i],b[i],b[i]])
        cdict = dict(red=red,green=green,blue=blue)
        #return cdict
        return mcolors.LinearSegmentedColormap(name=name,segmentdata=cdict)

    elif method == 'list' and ret_cmap_type == 'LinearSegmented':
        # generate list of values in the form of (value, c)
        outlist = []
        for i in range(len(x)):
            tup = (xNorm[i], (r[i],g[i],b[i]))
            outlist.append(tup)

        if N and type(N) == int:
        #return outlist
            return mcolors.LinearSegmentedColormap.from_list(name, outlist, N=N)
        else:
            raise TypeError("Using the method 'list' requires you to set a number of colors N.")

    elif ret_cmap_type == 'Listed':
        # generate list of values and return it as ListedColormap
        # returns both colors and the normalized positions (pos) where colors change, in the form of two outputs pos, colors
        pos_out = []
        colors_out = []
        for i in range(len(x)):
            pos_out.append(xNorm[i]) # list of positions
            colors_out.append(mcolors.to_hex( (r[i],g[i],b[i]))) # list of colors
        # return pos, color pairs
        print(colors_out)
        if N and type(N) == int and N<=len(colors_out):
            pos_out = pos_out[:N] #truncate positions to N
            return pos_out, mcolors.ListedColormap(colors_out, name=name, N=N)
        elif N is None:
            return pos_out, mcolors.ListedColormap(colors_out, name=name)
        else:
            raise TypeError("N has to be a number of colors that is less than the actual colors found in the .cpt file (" + str(len(colors_out)) + " colors found).")
    elif ret_cmap_type == 'raw':
        pos_out = []
        colors_out = []
        for i in range(len(x)):
            pos_out.append(xNorm[i]) # list of positions
            colors_out.append((r[i]*255, g[i]*255, b[i]*255))
        return pos_out, colors_out
    else:
        raise TypeError("method has to be one of the arguments: " + str(methodnames) + " and ret_cmap_type has to be one of the arguments: " + str(ret_cmap_types))

def plot_cmaps(cmap_list, width=6, cmap_height=0.5, axes_off=False):
    """Plot a colormap or list of colormaps with their names.
    Parameters
    -------
    cmap_list (str, cmap object or list of cmap objects anr strings): a list of colormaps to plot, either as cmap objects OR as preinstalled matplotlib colormap strings
    width (float): width of plot
    cmap_height (float): height of each colormap in plot
    axes_off (bool): boolean to erase axes

    Returns
    -------
    a matplotlib figure object (matplotlib.figure.Figure)

    Credits
    -------
    This function originally appears in pycpt, slight edits from bouziot, 2020.03
    https://github.com/j08lue/pycpt
    http://matplotlib.org/examples/color/colormaps_reference.html
    """
    if not isinstance(cmap_list, list):
        cmap_list = [cmap_list] # make subscriptable

    gradient = np.linspace(0, 1, 256)
    gradient = np.vstack((gradient, gradient))

    fig, axes = plt.subplots(nrows=len(cmap_list), figsize=(width,cmap_height*len(cmap_list)))
    fig.subplots_adjust(top=1, bottom=0, left=0, right=0.9)

    if len(cmap_list) == 1:
        cmap = cmap_list[0]
        # CASE ONE CMAP
        if isinstance(cmap, str):
                cmap = plt.get_cmap(cmap)
        axes.imshow(gradient, aspect='auto', cmap=cmap)
        pos = list(axes.get_position().bounds)
        x_text = pos[0] + pos[2] + 0.02
        y_text = pos[1] + pos[3]/2.
        fig.text(x_text, y_text, cmap.name, va='center', ha='left', fontsize=12)
        if axes_off:
            axes.set_axis_off()
        return fig

    else:
        # CASE MULTIPLE CMAPS
        for i, cmap in enumerate(cmap_list):
            if isinstance(cmap, str):
                cmap = plt.get_cmap(cmap)
            axes[i].imshow(gradient, aspect='auto', cmap=cmap)
            pos = list(axes[i].get_position().bounds)
            x_text = pos[0] + pos[2] + 0.02
            y_text = pos[1] + pos[3]/2.
            fig.text(x_text, y_text, cmap.name, va='center', ha='left', fontsize=12)
        # Turn off *all* ticks & spines, not just the ones with colormaps.
        if axes_off:
            for ax in axes:
                ax.set_axis_off()
        return fig

if __name__ == '__main__':
    #tests
    # test 1: FULL PATH, LinearSegmented, method cdict
    cpt_path = r'D:\Users\bouzidi\Desktop\matplotlib colormaps - cpt-city\cpt'
    cpt_fullpath = os.path.join(cpt_path, 'mby.cpt')

    a = get_cmap(cpt_fullpath)
    print(a)

    # test 2: LOCAL FILE, CHANGE BASEDIR
    basedir = r'D:\Users\bouzidi\Desktop\matplotlib colormaps - cpt-city\test\new_ctp'
    print(basedir)
    myctp2 = 'purple-orange-d15.cpt'
    pos, b = get_listed_cmap(myctp2)

    # test 3: url
    myurl = 'http://soliton.vm.bytemark.co.uk/pub/cpt-city/km/purple-orange-d15.cpt'
    c = get_cmap(myurl)
    print(c)
    print(c.name)

    # test 4: plots
    fig = plot_cmaps([a,b,c])
    plt.show()


import sys
import os
import argparse
# matplotlib.use("Qt5Agg")
from PyQt5.QtGui import QIcon, QKeySequence
from PyQt5.QtWidgets import QApplication, QMainWindow, QVBoxLayout, \
                            QSizePolicy, QWidget, QDesktopWidget, \
                            QPushButton, QHBoxLayout, QFileDialog, \
                            QAction, QShortcut
from os.path import exists, dirname, join
from seispy.setuplog import setuplog
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib.pyplot as plt



class SFigure(Figure):
    def __init__(self, eqs, para, width=21, height=11, dpi=100):
        self.eqs = eqs
        self.row_num = eqs.shape[0]
        self.para = para
        self.log = setuplog()
        self.idx = 0
        self.init_figure(width=width, height=height, dpi=dpi)
        self.plot()
        self.drop_lst = []
        self.drop_color = 'lightgray'

    def init_figure(self, width=21, height=11, dpi=100):
        self.fig, self.axs = plt.subplots(3, 1, sharex=True, figsize=(width, height), dpi=dpi, tight_layout=True)
    
    def set_properties(self):
        date = self.eqs.iloc[self.idx]['date'].strftime("%Y.%m.%dT%H:%M:%S")
        dis = self.eqs.iloc[self.idx]['dis']
        bazi = self.eqs.iloc[self.idx]['bazi']
        mag = self.eqs.iloc[self.idx]['mag']
        title = '({}/{}) {}, Baz: {:.2f}$^{{\circ}}$, Distance:{:.2f} km, Mw: {:.1f}'.format(self.idx+1, self.row_num, date, bazi, dis, mag)
        self.fig.suptitle(title)
        for ax in self.axs:
            ax.set(xlim=[-self.para.time_before, self.para.time_after])
            ax.minorticks_on()
        self.axs[2].set_xlabel('Time (s)')

    def clear(self):
        for ax in self.axs:
            ax.cla()

    def plot(self, lc='tab:blue'):
        st = self.eqs.iloc[self.idx]['data'].rf
        self.time_axis = st[0].times()-self.para.time_before
        for i in range(3):
            self.axs[i].plot(self.time_axis, st[i].data, color=lc)
            self.axs[i].axvline(x=0, lw=1, ls='--', color='r')
            self.axs[i].set_ylabel(st[i].stats.channel)
        self.set_properties()

    def next_action(self):
        self.clear()
        self.idx += 1
        if self.idx >= self.row_num:
            self.idx = 0
        if self.eqs.index[self.idx] in self.drop_lst:
            self.plot(lc=self.drop_color)
        else:
            self.plot()

    def back_action(self):
        self.clear()
        self.idx -= 1
        if self.idx < 0 :
            self.idx = 0
        if self.eqs.index[self.idx] in self.drop_lst:
            self.plot(lc=self.drop_color)
        else:
            self.plot()

    def drop(self):
        if self.eqs.index[self.idx] in self.drop_lst:
            return
        self.drop_lst.append(self.eqs.index[self.idx])
        self.plot(lc=self.drop_color)

    def cancel(self):
        if self.eqs.index[self.idx] not in self.drop_lst:
            return
        self.drop_lst.remove(self.eqs.index[self.idx])
        self.plot()

    def finish(self):
        self.eqs.drop(self.drop_lst, inplace=True)

class MyMplCanvas(FigureCanvas):
    def __init__(self, parent=None, eqs=None, para=None, width=21, height=11, dpi=100):

        plt.rcParams['axes.unicode_minus'] = False 
        self.sfig = SFigure(eqs, para, width=width, height=height, dpi=dpi)

        FigureCanvas.__init__(self, self.sfig.fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Expanding,
                                   QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

class MatplotlibWidget(QMainWindow):
    def __init__(self, eqs, para, parent=None):
        super(MatplotlibWidget, self).__init__(parent)
        self.initUi(eqs, para)

    def initUi(self, eqs, para):
        self.mpl = MyMplCanvas(self, eqs=eqs, para=para, width=21, height=11, dpi=100)
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.mpl, 2)

        main_frame = QWidget()
        self.setCentralWidget(main_frame)
        main_frame.setLayout(self.layout)

        self._set_geom_center()
        self._define_global_shortcuts()
        self.setWindowTitle('PickSPhease')
        self.setWindowIcon(QIcon(join(dirname(__file__), 'data', 'seispy.png')))

    def exit_app(self):
        self.close()

    def next_connect(self):
        self.mpl.sfig.next_action()
        self.mpl.draw()
    
    def back_connect(self):
        self.mpl.sfig.back_action()
        self.mpl.draw()
    
    def drop_connect(self):
        self.mpl.sfig.drop()
        self.mpl.draw()
    
    def cancel_connect(self):
        self.mpl.sfig.cancel()
        self.mpl.draw()
    
    def finish_connect(self):
        self.mpl.sfig.finish()
        QApplication.quit()
    
    def _define_global_shortcuts(self):
        self.key_c = QShortcut(QKeySequence('c'), self)
        self.key_c.activated.connect(self.next_connect)
        self.key_z = QShortcut(QKeySequence('z'), self)
        self.key_z.activated.connect(self.back_connect)
        self.key_d = QShortcut(QKeySequence('d'), self)
        self.key_d.activated.connect(self.drop_connect)
        self.key_a = QShortcut(QKeySequence('a'), self)
        self.key_a.activated.connect(self.cancel_connect)
        self.key_enter = QShortcut(QKeySequence('Return'), self)
        self.key_enter.activated.connect(self.finish_connect)

    def _set_geom_center(self, height=0.7, width=1):
        screen_resolution = QDesktopWidget().screenGeometry()
        screen_height = screen_resolution.height()
        screen_width = screen_resolution.width()
        frame_height = int(screen_height * height)
        frame_width = int(screen_width * width)

        self.setGeometry(0, 0, frame_width, frame_height)
        self.move((screen_width / 2) - (self.frameSize().width() / 2),
                  (screen_height / 2) - (self.frameSize().height() / 2))


import glob
import os
import obspy
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from os.path import join, basename
from seispy.setuplog import setuplog
from seispy.plotRT import init_figure, set_fig, plot_waves


def indexpags(evt_num, maxidx=20):
    full_pages = evt_num//maxidx
    if np.mod(evt_num, maxidx) == 0:
        axpages = full_pages
    else:
        axpages = full_pages+1
    rfidx = []
    for i in range(axpages-1):
        rfidx.append(np.arange(maxidx*i, maxidx*(i+1)))
    rfidx.append(np.arange(maxidx*(axpages-1), evt_num))
    return axpages, rfidx


class StaData():
    def __init__(self, filenames, rrf, trf, baz, goodrf):
        goodidx = np.where(goodrf == 1)
        self.event = [filenames[i] for i in goodidx[0]]
        self.bazi = baz[goodidx]
        self.rflength = len(rrf[0].data)
        self.ev_num = len(self.bazi)
        self.datar = np.empty([self.ev_num, self.rflength])
        self.datat = np.empty([self.ev_num, self.rflength])
        for i, idx in enumerate(goodidx[0]):
            self.datar[i, :] = rrf[idx].data
            self.datat[i, :] = trf[idx].data
        self.time_axis = np.array([])


class RFFigure(Figure):
    def __init__(self, rfpath, width=21, height=11, dpi=100, xlim=[-2, 30]):
        super(RFFigure, self).__init__()
        self.width = width
        self.height = height
        self.dpi = dpi
        self.log = setuplog()
        self.rfpath = rfpath
        self.enf = 3.5
        self.ipage = 0
        self.maxidx = 20
        self.xlim = xlim

    def init_canvas(self):
        self.init_figure(width=self.width, height=self.height, dpi=self.dpi)
        self.read_sac()
        self.set_figure()
        self.set_page()
        self.init_variables()
        self.plotwave()
        self.plotbaz()

    def set_ylabels(self):
        self.axr.set_ylim(self.rfidx[self.ipage][0], self.rfidx[self.ipage][0]+self.maxidx+1)
        self.axr.set_yticks(np.arange(self.rfidx[self.ipage][0], self.rfidx[self.ipage][0]+self.maxidx+1))
        ylabels = np.array(self.filenames)[self.rfidx[self.ipage]]
        ticklabels = [''] * (self.maxidx+1)        
        ticklabels[1: len(ylabels)+1] = ylabels
        self.axr.set_yticklabels(ticklabels)
        self.axt.set_ylim(self.axr.get_ylim())
        self.axt.set_yticks(self.axr.get_yticks())
        self.axb.set_ylim(self.axr.get_ylim())
        self.axb.set_yticks(self.axr.get_yticks())
        self.azi_label = ['%5.2f' % self.baz[i] for i in self.rfidx[self.ipage]]
        ticklabels = [''] * (self.maxidx+1)
        ticklabels[1: len(self.azi_label)+1] = self.azi_label
        self.axb.set_yticklabels(ticklabels)

    def set_page(self):
        self.set_ylabels()
        self.axr.plot([0, 0], [0, self.axpages*self.maxidx+1], color="black")
        self.axt.plot([0, 0], [0, self.axpages*self.maxidx+1], color="black")
        self.axr.set_xlim(self.xlim[0], self.xlim[1])
        self.axr.set_xticks(np.arange(self.xlim[0], self.xlim[1]+1, 2))
        self.axt.set_xlim(self.xlim[0], self.xlim[1])
        self.axt.set_xticks(np.arange(self.xlim[0], self.xlim[1]+1, 2))
        self.axb.set_xlim(0, 360)
        self.axb.set_xticks(np.arange(0, 361, 60))
        self.axg.set_xlim(30, 90)
        self.axg.set_xticks(np.arange(30, 91, 10))

    def init_variables(self):
        self.goodrf = np.ones(self.evt_num)
        self.rlines = [[] for i in range(self.evt_num)]
        self.tlines = [[] for i in range(self.evt_num)]
        self.rwvfillpos = [[] for i in range(self.evt_num)]
        self.rwvfillnag = [[] for i in range(self.evt_num)]
        self.twvfillpos = [[] for i in range(self.evt_num)]
        self.twvfillnag = [[] for i in range(self.evt_num)]

    def init_figure(self, width=21, height=11, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi) 
        self.axr = self.fig.add_axes([0.1, 0.05, 0.35, 0.84])
        self.axt = self.fig.add_axes([0.47, 0.05, 0.35, 0.84])
        self.axb = self.fig.add_axes([0.855, 0.05, 0.12, 0.84])
        self.axg = self.axb.twiny()

    def set_figure(self):
        self.axr.grid(b=True, which='major', axis='x')
        self.axt.grid(b=True, which='major', axis='x')
        self.axb.grid(b=True, which='major')
        self.axr.set_ylabel("Event")
        self.axr.set_xlabel("Time after P (s)")
        self.axr.set_title("{} component".format(self.comp))
        self.axt.set_xlabel("Time after P (s)")
        self.axt.set_title("T component")
        self.axb.set_xlabel("Backazimuth (\N{DEGREE SIGN})")
        self.axg.set_xlabel('Distance (\N{DEGREE SIGN})')

    def read_sac(self, dt=0.1):
        if len(glob.glob(join(self.rfpath, '*_R.sac'))) == 0:
            tmp_files = glob.glob(join(self.rfpath, '*_Q.sac'))
            self.comp = 'Q'
        else:
            tmp_files = glob.glob(join(self.rfpath, '*_R.sac'))  
            self.comp = 'R'  
        self.log.RFlog.info('Reading PRFs from {}'.format(self.rfpath))
        self.filenames = [basename(sac_file).split('_')[0] for sac_file in sorted(tmp_files)]
        self.rrf = obspy.read(join(self.rfpath, '*_{}.sac'.format(self.comp))).sort(['starttime']).resample(1/dt)
        self.trf = obspy.read(join(self.rfpath, '*_T.sac')).sort(['starttime']).resample(1/dt)
        self.time_axis = self.rrf[0].times() + self.rrf[0].stats.sac.b
        self.evt_num = len(self.rrf)
        self.log.RFlog.info('A total of {} PRFs loaded'.format(self.evt_num))
        self.baz = np.array([tr.stats.sac.baz for tr in self.rrf])
        self.sort_baz_()

        self.axpages, self.rfidx = indexpags(self.evt_num, self.maxidx)
        self.staname = (self.rrf[0].stats.network+'.'+self.rrf[0].stats.station).strip('.')
        self.fig.suptitle("%s (Latitude: %5.2f\N{DEGREE SIGN}, Longitude: %5.2f\N{DEGREE SIGN})" % (self.staname, self.rrf[0].stats.sac.stla, self.rrf[0].stats.sac.stlo), fontsize=20)

    def sort_baz_(self):
        idx = np.argsort(self.baz)
        self.baz = self.baz[idx]
        self.rrf = [self.rrf[i] for i in idx]
        self.trf = [self.trf[i] for i in idx]
        self.gcarc = [self.rrf[i].stats.sac.gcarc for i in idx]
        self.filenames = [self.filenames[i] for i in idx]

    def plotwave(self):
        bound = np.zeros(self.time_axis.shape[0])
        for i in range(self.evt_num):
            r_amp_axis = self.rrf[i].data*self.enf+i+1
            t_amp_axis = self.trf[i].data*self.enf+i+1
            self.rlines[i], = self.axr.plot(self.time_axis, r_amp_axis, color="black", linewidth=0.2)            
            self.tlines[i], = self.axt.plot(self.time_axis, t_amp_axis, color="black", linewidth=0.2)
            self.rwvfillpos[i] = self.axr.fill_between(self.time_axis, r_amp_axis, bound+i+1, where=r_amp_axis >i+1, facecolor='red', alpha=0.3)
            self.rwvfillnag[i] = self.axr.fill_between(self.time_axis, r_amp_axis, bound+i+1, where=r_amp_axis <i+1, facecolor='blue', alpha=0.3)
            self.twvfillpos[i] = self.axt.fill_between(self.time_axis, t_amp_axis, bound+i+1, where=t_amp_axis >i+1, facecolor='red', alpha=0.3)
            self.twvfillnag[i] = self.axt.fill_between(self.time_axis, t_amp_axis, bound+i+1, where=t_amp_axis <i+1, facecolor='blue', alpha=0.3)

    def plotbaz(self):
        self.axb.scatter(self.baz, np.arange(self.evt_num)+1)
        self.axg.scatter(self.gcarc, np.arange(self.evt_num)+1, color='orange', alpha=0.6)

    def onclick(self, event):
        if event.inaxes != self.axr and event.inaxes != self.axt:
            return
        click_idx = int(np.round(event.ydata))
        if click_idx > self.evt_num:
            return
        if self.goodrf[click_idx-1] == 1:
            self.log.RFlog.info("Selected "+self.filenames[click_idx-1])
            self.goodrf[click_idx-1] = 0
            self.rwvfillpos[click_idx-1].set_facecolor('gray')
            self.rwvfillnag[click_idx-1].set_facecolor('gray')
            self.twvfillpos[click_idx-1].set_facecolor('gray')
            self.twvfillnag[click_idx-1].set_facecolor('gray')
        else:
            self.log.RFlog.info("Canceled "+self.filenames[click_idx-1])
            self.goodrf[click_idx-1] = 1
            self.rwvfillpos[click_idx-1].set_facecolor('red')
            self.rwvfillnag[click_idx-1].set_facecolor('blue')
            self.twvfillpos[click_idx-1].set_facecolor('red')
            self.twvfillnag[click_idx-1].set_facecolor('blue')

    def _set_gray(self):
        for i in np.where(self.goodrf == 0)[0]:
            self.rwvfillpos[i].set_facecolor('gray')
            self.rwvfillnag[i].set_facecolor('gray')
            self.twvfillpos[i].set_facecolor('gray')
            self.twvfillnag[i].set_facecolor('gray')

    def butprevious(self):
        self.ipage -= 1
        if self.ipage < 0:
            self.ipage = 0
            return
        self.set_ylabels()
        self.set_figure()

    def butnext(self):
        self.ipage += 1
        if self.ipage >= self.axpages:
            self.ipage = self.axpages-1
            return
        self.set_ylabels()
        self.set_figure()

    def enlarge(self):
        self.enf += 1
        self.axr.cla()
        self.axt.cla()
        self.plotwave()
        self._set_gray()
        self.set_page()
        self.set_figure()

    def reduce(self):
        if self.enf > 1:
            self.enf -= 1
        else:
            self.enf = 1/(1/self.enf + 1)
        self.axr.cla()
        self.axt.cla()
        self.plotwave()
        self._set_gray()
        self.set_page()
        self.set_figure()

    def finish(self):
        badidx = np.where(self.goodrf == 0)[0]
        self.log.RFlog.info("%d RFs are rejected" % len(badidx))
        with open(os.path.join(self.rfpath, self.staname+"finallist.dat"), 'w+') as fid:
            for i in range(self.evt_num):
                if self.goodrf[i] == 0:
                    files = glob.glob(join(self.rfpath, self.filenames[i]+'*.sac'))
                    for fil in files:
                        os.remove(fil)
                    self.log.RFlog.info("Reject PRF of "+self.filenames[i])
                else:
                    evla = self.rrf[i].stats.sac.evla
                    evlo = self.rrf[i].stats.sac.evlo
                    evdp = self.rrf[i].stats.sac.evdp
                    dist = self.rrf[i].stats.sac.gcarc
                    baz = self.rrf[i].stats.sac.baz
                    rayp = self.rrf[i].stats.sac.user0
                    mag = self.rrf[i].stats.sac.mag
                    gauss = self.rrf[i].stats.sac.user1
                    fid.write('%s %s %6.3f %6.3f %6.3f %6.3f %6.3f %8.7f %6.3f %6.3f\n' % (self.filenames[i], 'P', evla, evlo, evdp, dist, baz, rayp, mag, gauss))

    def plot(self):
        plt.ion()
        plt.rcParams['toolbar'] = 'None'
        stadata = StaData(self.filenames, self.rrf, self.trf, self.baz, self.goodrf)
        stadata.time_axis = self.time_axis
        self.plotfig, axr, axt, axb, axr_sum, axt_sum = init_figure()
        plot_waves(axr, axt, axb, axr_sum, axt_sum, stadata, enf=self.enf)
        set_fig(axr, axt, axb, axr_sum, axt_sum, stadata, self.staname)



from os.path import expanduser
import configparser
import numpy as np
from seispy.utils import check_path


class HKPara(object):
    def __init__(self):
        self.rfpath = expanduser('~')
        self.hkpath = expanduser('~')
        self.hklist = 'hk.dat'
        self.hrange = np.arange(20, 80, 0.1)
        self.krange = np.arange(1.6, 1.9, 0.01)
        self.vp = 6.3
        self.weight = (0.7, 0.2, 0.1)

    @property
    def hrange(self):
        return self._hrange

    @hrange.setter
    def hrange(self, value):
        if not (isinstance(value, np.ndarray) or value is None):
            raise TypeError('Error type of hrange')
        else:
            self._hrange = value

    @property
    def krange(self):
        return self._krange

    @krange.setter
    def krange(self, value):
        if not (isinstance(value, np.ndarray) or value is None):
            raise TypeError('Error type of krange')
        else:
            self._krange = value


def hkpara(cfg_file):
    hpara = HKPara()
    cf = configparser.ConfigParser()
    try:
        cf.read(cfg_file)
    except Exception:
        raise FileNotFoundError('Cannot open configure file %s' % cfg_file)

    # para for FileIO section
    hpara.rfpath = check_path('rfpath', cf.get('FileIO', 'rfpath'))
    hpara.hkpath = cf.get('FileIO', 'hkpath')
    hpara.hklist = cf.get('FileIO', 'hklst')

    hmin = cf.getfloat('hk', 'hmin')
    hmax = cf.getfloat('hk', 'hmax')
    kmin = cf.getfloat('hk', 'kmin')
    kmax = cf.getfloat('hk', 'kmax')
    hpara.hrange = np.arange(hmin, hmax, 0.1)
    hpara.krange = np.arange(kmin, kmax, 0.01)

    vp = cf.get('hk', 'vp')
    if vp != '':
        hpara.vp = float(vp)

    w1 = cf.getfloat('hk', 'weight1')
    w2 = cf.getfloat('hk', 'weight2')
    w3 = cf.getfloat('hk', 'weight3')
    hpara.weight = (w1, w2, w3)
    return hpara



import numpy as np
from os.path import expanduser, join, dirname, exists
import configparser
from seispy.geo import km2deg
from seispy.utils import check_path


class CCPPara(object):
    def __init__(self):
        self.rfpath = expanduser('~')
        self.rayp_lib = None
        self.depthdat = 'RFdepth.mat'
        self.stackfile = 'ccp.dat'
        self.stalist = 'sta.lst'
        self.peakfile = 'good_410_660.dat'
        self.velmod = ''
        self.stack_sta_list = ''
        self.domperiod = 5
        self.shape = 'circle'
        self.slide_val = 5
        self.width = 100
        self.bin_radius = 50
        self.line = np.array([])
        self.depth_axis = np.array([])
        self.stack_range = np.array([])
        self.center_bin = []
        self.dep_val = 1
        self.stack_val = 1
        self.boot_samples = None
        
    @property
    def bin_radius(self):
        return self._bin_radius

    @bin_radius.setter
    def bin_radius(self, value):
        if not (isinstance(value, (int, float)) or value is None):
            raise TypeError('Error type of bin_radius')
        else:
            self._bin_radius = value

    @property
    def shape(self):
        return self._shape

    @shape.setter
    def shape(self, value):
        if not isinstance(value, str):
            raise TypeError('ccppara.shape must be str type')
        elif value.lower() not in ('circle', 'rect'):
            raise ValueError('ccppara.shape must be in \'circle\' or \'rect\'')
        else:
            self._shape = value.lower()


def ccppara(cfg_file):
    cpara = CCPPara()
    cf = configparser.ConfigParser()
    try:
        cf.read(cfg_file)
    except Exception:
        raise FileNotFoundError('Cannot open configure file %s' % cfg_file)
    # para for FileIO section
    cpara.rfpath = cf.get('FileIO', 'rfpath')
    rayp_lib = cf.get('FileIO', 'rayp_lib')
    if rayp_lib == '':
        cpara.rayp_lib = None
    else:
        cpara.rayp_lib = rayp_lib
    cpara.depthdat = cf.get('FileIO', 'depthdat')
    cpara.stackfile = cf.get('FileIO', 'stackfile')
    cpara.stalist = cf.get('FileIO', 'stalist')
    cpara.stack_sta_list = cf.get('FileIO', 'stack_sta_list')
    if cf.has_option('FileIO', 'peakfile'):
        fname = cf.get('FileIO', 'peakfile')
        if fname != '':
            cpara.peakfile = fname
    velmod = cf.get('FileIO', 'velmod')
    if velmod == '':
        cpara.velmod = join(dirname(__file__), 'data', 'iasp91.vel')
    elif not exists(velmod):
        cpara.velmod = join(dirname(__file__), 'data', '{}.vel'.format(velmod.lower()))
    else:
        cpara.velmod = velmod
    # para for bin section
    cpara.shape = cf.get('bin', 'shape')
    try:
        cpara.domperiod = cf.getfloat('bin', 'domperiod')
    except:
        cpara.domperiod = None
    try:
        cpara.width = cf.getfloat('bin', 'width')
    except:
        cpara.width = None
    try:
        cpara.bin_radius = cf.getfloat('bin', 'bin_radius')
    except:
        cpara.bin_radius = None
    cpara.slide_val = cf.getfloat('bin', 'slide_val')
    # para for line section
    lat1 = cf.getfloat('line', 'profile_lat1')
    lon1 = cf.getfloat('line', 'profile_lon1')
    lat2 = cf.getfloat('line', 'profile_lat2')
    lon2 = cf.getfloat('line', 'profile_lon2')
    cpara.line = np.array([lat1, lon1, lat2, lon2])
    # para for depth section
    dep_end = cf.getfloat('depth', 'dep_end')
    cpara.dep_val = cf.getfloat('depth', 'dep_val')
    cpara.depth_axis = np.append(np.arange(0, dep_end, cpara.dep_val), dep_end)

    stack_start = cf.getfloat('stack', 'stack_start')
    stack_end = cf.getfloat('stack', 'stack_end')
    cpara.stack_val = cf.getfloat('stack', 'stack_val')
    cpara.stack_range = np.arange(stack_start, stack_end, cpara.stack_val)
    try:
        cpara.boot_samples = cf.getint('stack', 'boot_samples')
    except:
        cpara.boot_samples = None
    # para for center bins
    if cf.has_section('spacedbins'):
        cla = cf.getfloat('spacedbins', 'center_lat')
        clo = cf.getfloat('spacedbins', 'center_lon')
        hlla = cf.getfloat('spacedbins', 'half_len_lat')
        hllo = cf.getfloat('spacedbins', 'half_len_lon')
        cpara.center_bin = [cla, clo, hlla, hllo, km2deg(cpara.slide_val)]

    return cpara



import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from matplotlib.lines import Line2D
from seispy.rfcorrect import SACStation
from seispy.rf import CfgParser
import argparse
import numpy as np
from os.path import join, realpath, basename, exists
import sys


def init_figure():
    h = plt.figure(figsize=(11.7, 8.3))
    gs = GridSpec(17, 3)
    gs.update(wspace=0.25)
    axr_sum = plt.subplot(gs[0, 0])
    axr_sum.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    axr = plt.subplot(gs[1:, 0])
    axr.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    axt_sum = plt.subplot(gs[0, 1])
    axt_sum.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    axt = plt.subplot(gs[1:, 1])
    axt.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    axb = plt.subplot(gs[1:, 2])
    axb.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    return h, axr, axt, axb, axr_sum, axt_sum


def read_process_data(path, resamp_dt=0.1):
    stadata = SACStation(path)
    stadata.resample(resamp_dt)
    idx = np.argsort(stadata.bazi)
    stadata.event = stadata.event[idx]
    stadata.bazi = stadata.bazi[idx]
    stadata.datar = stadata.datar[idx]
    stadata.datat = stadata.datat[idx]
    return stadata


def plot_waves(axr, axt, axb, axr_sum, axt_sum, stadata, enf=3):
    bound = np.zeros(stadata.rflength)
    for i in range(stadata.ev_num):
        datar = stadata.datar[i] * enf + (i + 1)
        datat = stadata.datat[i] * enf + (i + 1)
        # axr.plot(time_axis, stadata.datar[i], linewidth=0.2, color='black')
        axr.fill_between(stadata.time_axis, datar, bound + i+1, where=datar > i+1, facecolor='red',
                         alpha=0.7)
        axr.fill_between(stadata.time_axis, datar, bound + i+1, where=datar < i+1, facecolor='blue',
                         alpha=0.7)
        # axt.plot(time_axis, stadata.datat[i], linewidth=0.2, color='black')
        axt.fill_between(stadata.time_axis, datat, bound + i + 1, where=datat > i+1, facecolor='red',
                         alpha=0.7)
        axt.fill_between(stadata.time_axis, datat, bound + i + 1, where=datat < i+1, facecolor='blue',
                         alpha=0.7)
    datar = np.mean(stadata.datar, axis=0)
    datar /= np.max(datar)
    datat = np.mean(stadata.datat, axis=0)
    datat /= np.max(datar)
    axr_sum.fill_between(stadata.time_axis, datar, bound, where=datar > 0, facecolor='red', alpha=0.7)
    axr_sum.fill_between(stadata.time_axis, datar, bound, where=datar < 0, facecolor='blue', alpha=0.7)
    axt_sum.fill_between(stadata.time_axis, datat, bound, where=datat > 0, facecolor='red', alpha=0.7)
    axt_sum.fill_between(stadata.time_axis, datat, bound, where=datat < 0, facecolor='blue', alpha=0.7)
    axb.scatter(stadata.bazi, np.arange(stadata.ev_num) + 1, s=7)


def set_fig(axr, axt, axb, axr_sum, axt_sum, stadata, station, xmin=-2, xmax=30, comp='R'):
    y_range = np.arange(stadata.ev_num) + 1
    x_range = np.arange(0, xmax+2, 2)
    space = 2

    # set axr
    axr.set_xlim(xmin, xmax)
    axr.set_xticks(x_range)
    axr.set_xticklabels(x_range, fontsize=8)
    axr.set_ylim(0, stadata.ev_num + space)
    axr.set_yticks(y_range)
    axr.set_yticklabels(stadata.event, fontsize=5)
    axr.set_xlabel('Time after P (s)', fontsize=13)
    axr.set_ylabel('Event', fontsize=13)
    axr.add_line(Line2D([0, 0], axr.get_ylim(), color='black'))

    # set axr_sum
    axr_sum.set_title('{} components ({})'.format(comp, station), fontsize=16)
    axr_sum.set_xlim(xmin, xmax)
    axr_sum.set_xticks(x_range)
    axr_sum.set_xticklabels([])
    axr_sum.set_ylim(-0.5, 1.25)
    axr_sum.set_yticks([0.375])
    axr_sum.set_yticklabels(['Sum'], fontsize=8)
    axr_sum.tick_params(axis='y', left=False)
    axr_sum.add_line(Line2D([0, 0], axr_sum.get_ylim(), color='black'))

    # set axt
    axt.set_xlim(xmin, xmax)
    axt.set_xticks(x_range)
    axt.set_xticklabels(x_range, fontsize=8)
    axt.set_ylim(0, stadata.ev_num + space)
    axt.set_yticks(y_range)
    bazi = ['{:.1f}'.format(ba) for ba in stadata.bazi]
    axt.set_yticklabels(bazi, fontsize=5)
    axt.set_xlabel('Time after P (s)', fontsize=13)
    axt.set_ylabel(r'Back-azimuth ($\circ$)', fontsize=13)
    axt.add_line(Line2D([0, 0], axt.get_ylim(), color='black'))

    # set axt_sum
    axt_sum.set_title('T components ({})'.format(station), fontsize=16)
    axt_sum.set_xlim(xmin, xmax)
    axt_sum.set_xticks(x_range)
    axt_sum.set_xticklabels([])
    axt_sum.set_ylim(-0.5, 1.25)
    axt_sum.set_yticks([0.375])
    axt_sum.set_yticklabels(['Sum'], fontsize=8)
    axt_sum.tick_params(axis='y', left=False)
    axt_sum.add_line(Line2D([0, 0], axt_sum.get_ylim(), color='black'))

    # set axb
    axb.set_xlim(0, 360)
    axb.set_xticks(np.linspace(0, 360, 7))
    axb.set_xticklabels(np.linspace(0, 360, 7, dtype='i'), fontsize=8)
    axb.set_ylim(0, stadata.ev_num + space)
    axb.set_yticks(y_range)
    axb.set_yticklabels(y_range, fontsize=5)
    axb.set_xlabel(r'Back-azimuth ($\circ$)', fontsize=13)


def plotrt(rfpath, enf=3, out_path='./', outformat='g'):
    """[summary]

    :param rfpath: [description]
    :type rfpath: [type]
    :param enf: [description], defaults to 3
    :type enf: int, optional
    :param out_path: [description], defaults to './'
    :type out_path: str, optional
    :param outformat: [description], defaults to 'g'
    :type outformat: str, optional
    :raises FileExistsError: [description]
    :raises FileExistsError: [description]
    """
    station = basename(rfpath)
    lst = join(rfpath, station+'finallist.dat')
    if not exists(lst):
        raise FileExistsError('No such a final list as {}'.format(lst))
    if not exists(out_path):
        raise FileExistsError('The output path {} not exists'.format(out_path))
    h, axr, axt, axb, axr_sum, axt_sum = init_figure()
    stadata = read_process_data(rfpath)
    plot_waves(axr, axt, axb, axr_sum, axt_sum, stadata, enf=enf)
    set_fig(axr, axt, axb, axr_sum, axt_sum, stadata, station)
    if outformat == 'g':
        h.savefig(join(out_path, station+'_RT_bazorder_{:.1f}.png'.format(stadata.f0[0])), dpi=400, bbox_inches='tight')
    elif outformat == 'f':
        h.savefig(join(out_path, station+'_RT_bazorder_{:.1f}.pdf'.format(stadata.f0[0])), format='pdf', bbox_inches='tight')


def main():
    parser = argparse.ArgumentParser(description="Plot R(Q)&T components for P receiver functions (PRFs)")
    parser.add_argument('rfpath', help='Path to PRFs with a \'finallist.dat\' in it', type=str, default=None)
    parser.add_argument('-e', help='Enlargement factor, defaults to 3', dest='enf', type=float, default=3, metavar='enf')
    parser.add_argument('-o', help='Output path without file name, defaults to current path', dest='output', default='./', type=str, metavar='outpath')
    parser.add_argument('-t', help='Specify figure format. f = \'.pdf\', g = \'.png\', defaults to \'g\'',
                        dest='format', default='g', type=str, metavar='f|g')
    arg = parser.parse_args()
    if arg.format not in ('f', 'g'):
        print('Errpr: The format must be in \'f\' and \'g\'')
        parser.print_help()
        sys.exit(1)
    plotrt(rfpath=arg.path, enf=arg.enf, out_path=arg.output, outformat=arg.format)


if __name__ == '__main__':
    # plotrt('XHL01')
    pass



from obspy.io.sac.sactrace import SACTrace
import numpy as np
from scipy.interpolate import interp1d, interpn
from scipy.signal import resample
from os.path import dirname, join, exists, basename, isfile
from seispy.geo import skm2srad, sdeg2skm, rad2deg, latlon_from, \
                       asind, tand, srad2skm, km2deg
from seispy.psrayp import get_psrayp
from seispy.rfani import RFAni
import matplotlib.pyplot as plt
import warnings
import glob

warnings.filterwarnings("ignore")


class SACStationS():
    def __init__(self, path):
        fnames = sorted(glob.glob(join(path, '*S_*.sac')))
        self.ev_num = len(fnames)
        self.rayp = np.zeros(self.ev_num)
        self.bazi = np.zeros(self.ev_num)
        self.dis = np.zeros(self.ev_num)
        self.evla = np.zeros(self.ev_num)
        self.evlo = np.zeros(self.ev_num)
        self.evdp = np.zeros(self.ev_num)
        sample_sac = SACTrace.read(fnames[0])
        self.sampling = sample_sac.delta
        self.stla = sample_sac.stla
        self.stlo = sample_sac.stlo
        self.shift = -sample_sac.b
        self.RFlength = sample_sac.npts
        self.datal = np.zeros([self.ev_num, self.RFlength])
        for i, sacfile in enumerate(fnames):
            sac = SACTrace.read(sacfile)
            self.rayp[i] = sac.user0
            self.bazi[i] = sac.baz
            self.dis[i] = sac.gcarc
            self.evla[i] = sac.evla
            self.evlo[i] = sac.evlo
            self.evdp[i] = sac.evdp
            self.datal[i] = sac.data
        self.rayp = skm2srad(self.rayp)


class SACStation(object):
    def __init__(self, data_path, only_r=False):
        """
        Class for derivative process of RFs.

        :param data_path: Path to RF data with SAC format. A finallist.dat must be in this path.
        :type data_path: str
        :param only_r: [description], defaults to False
        :type only_r: bool, optional

        .. warning::

            This Class will be renamed to ``RFStation`` in future versions.
        """
        self.only_r = only_r
        if isfile(data_path):
            data_path = dirname(data_path)
        self.staname = basename(data_path)
        evt_lsts = glob.glob(join(data_path, '*finallist.dat'))
        if len(evt_lsts) == 0:
            raise FileNotFoundError("No such *finallist.dat in the {}".format(data_path))
        elif len(evt_lsts) > 1:
            raise ValueError("More than one finallist.dat in the {}".format(data_path))
        else:
            evt_lst = evt_lsts[0]
        dtype = {'names': ('evt', 'phase', 'evlat', 'evlon', 'evdep', 'dis', 'bazi', 'rayp', 'mag', 'f0'),
                 'formats': ('U20', 'U20', 'f4', 'f4', 'f4', 'f4', 'f4', 'f4', 'f4', 'f4')}
        self.event, self.phase, self.evla, self.evlo, self.evdp, self.dis, self.bazi, self.rayp, self.mag, self.f0 = \
            np.loadtxt(evt_lst, dtype=dtype, unpack=True, ndmin=1)
        # self.event = [datestr.decode() for datestr in self.event]
        # self.phase = [ph.decode() for ph in self.phase]
        self.rayp = skm2srad(self.rayp)
        self.ev_num = self.evla.shape[0]
        try:
            sample_sac = SACTrace.read(join(data_path, self.event[0] + '_' + self.phase[0] + '_R.sac'))
            self.comp = 'R'
        except:
            sample_sac = SACTrace.read(join(data_path, self.event[0] + '_' + self.phase[0] + '_Q.sac'))
            self.comp = 'Q'
        self.stla = sample_sac.stla
        self.stlo = sample_sac.stlo
        self.rflength = sample_sac.npts
        self.RFlength = self.rflength
        self.shift = -sample_sac.b
        self.sampling = sample_sac.delta
        self.time_axis = np.arange(self.rflength) * self.sampling - self.shift
        self.datar = np.empty([self.ev_num, self.rflength])
        if not only_r:
            self.datat = np.empty([self.ev_num, self.rflength])
            for _i, evt, ph in zip(range(self.ev_num), self.event, self.phase):
                sac = SACTrace.read(join(data_path, evt + '_' + ph + '_{}.sac'.format(self.comp)))
                sact = SACTrace.read(join(data_path, evt + '_' + ph + '_T.sac'))
                self.datar[_i] = sac.data
                self.datat[_i] = sact.data
        else:
            for _i, evt, ph in zip(range(self.ev_num), self.event, self.phase):
                sac = SACTrace.read(join(data_path, evt + '_' + ph + '_{}.sac'.format(self.comp)))
                self.datar[_i] = sac.data

    def normalize(self):
        maxamp = np.nanmax(np.abs(self.datar), axis=1)
        for i in range(self.ev_num):
            self.datar[i] /= maxamp[i]
            if not self.only_r:
                self.datat[i] /= maxamp[i]

    def resample(self, dt):
        npts = int(self.rflength * (self.sampling / dt)) + 1
        self.datar = resample(self.datar, npts, axis=1)
        if not self.only_r:
            self.datat = resample(self.datat, npts, axis=1)
        self.sampling = dt
        self.rflength = npts
        self.time_axis = np.arange(npts) * dt - self.shift

    def moveoutcorrect(self, ref_rayp=0.06, dep_range=np.arange(0, 150), velmod='iasp91'):
        rf_corr, _ = moveoutcorrect_ref(self, skm2srad(ref_rayp), dep_range, velmod=velmod)
        return rf_corr

    def psrf2depth(self, dep_range=np.arange(0, 150), velmod='iasp91', srayp=None):
        self.dep_range = dep_range
        rfdepth, _, _, _ = psrf2depth(self, dep_range, sampling=self.sampling, shift=self.shift, velmod=velmod, srayp=srayp)
        return rfdepth

    def psrf_1D_raytracing(self, dep_range=np.arange(0, 150), velmod='iasp91', srayp=None):
        self.dep_range = dep_range
        pplat_s, pplon_s, _ , _, _, _, tpds = psrf_1D_raytracing(self, dep_range, velmod=velmod, srayp=srayp)
        return pplat_s, pplon_s, tpds

    def psrf_3D_raytracing(self, mod3dpath, dep_range=np.arange(0, 150), srayp=None):
        self.dep_range = dep_range
        mod3d = Mod3DPerturbation(mod3dpath, dep_range)
        pplat_s, pplon_s, _, _, tpds = psrf_3D_raytracing(self, dep_range, mod3d, srayp=srayp)
        return pplat_s, pplon_s, tpds

    def psrf_3D_moveoutcorrect(self,  mod3dpath, dep_range=np.arange(0, 150), velmod='iasp91', srayp=None):
        self.dep_range = dep_range
        mod3d = Mod3DPerturbation(mod3dpath, dep_range)
        pplat_s, pplon_s, pplat_p, pplon_p, raylength_s, raylength_p, Tpds = psrf_1D_raytracing(self, dep_range, velmod=velmod, srayp=srayp)
        tps = psrf_3D_migration(pplat_s, pplon_s, pplat_p, pplon_p, raylength_s, raylength_p, Tpds, dep_range, mod3d)
        rfdepth, _ = time2depth(self, dep_range, tps)
        return rfdepth

    def jointani(self, tb, te, tlen=3, stack_baz_val=10, weight=[0.4, 0.4, 0.2]):
        self.ani = RFAni(self, tb, te, tlen=tlen)
        self.ani.baz_stack(val=stack_baz_val)
        best_f, best_t = self.ani.joint_ani(weight=weight)
        return best_f, best_t


class RFStation(SACStation):
    def __init__(self, data_path, only_r):
        """Class for derivative process of RFs.

        :param data_path: Path to RF data with SAC format. A finallist.dat must be in this path.
        :type data_path: str
        :param only_r: [description], defaults to False
        :type only_r: bool, optional
        """
        super().__init__(data_path, only_r=only_r)


class DepModel(object):
    def __init__(self, YAxisRange, velmod='iasp91'):
        VelocityModel = np.loadtxt(from_file(velmod))
        self.depthsraw = VelocityModel[:, 0]
        self.vpraw = VelocityModel[:, 1]
        self.vsraw = VelocityModel[:, 2]
        self.vp = interp1d(self.depthsraw, self.vpraw, bounds_error=False, fill_value='extrapolate')(YAxisRange)
        self.vs = interp1d(self.depthsraw, self.vsraw, bounds_error=False, fill_value='extrapolate')(YAxisRange)
        self.depths = YAxisRange
        self.dz = np.append(0, np.diff(self.depths))
        self.R = 6371 - self.depths


def _imag2nan(arr):
    StopIndex = np.where(np.imag(arr) == 1)[0]
    if StopIndex.size != 0:
        arr[StopIndex[0]:] = np.nan
    return arr


def from_file(mode_name):
    if exists(mode_name):
        filename = mode_name
    elif exists(join(dirname(__file__), 'data', mode_name.lower()+'.vel')):
        filename = join(dirname(__file__), 'data', mode_name.lower()+'.vel')
    else:
        raise ValueError('No such velocity mode')
    return filename


def moveoutcorrect_ref(stadatar, raypref, YAxisRange, sampling=None, shift=None, velmod='iasp91'):
    """
    :param stadatar: data class of SACStation
    :param raypref: referred ray parameter in rad
    :param YAxisRange: Depth range in nd.array type
    :param velmod: Path to velocity model

    :return: Newdatar, EndIndex
    """
    sampling = stadatar.sampling
    shift = stadatar.shift
    if 'datar' in stadatar.__dict__:
        data = stadatar.datar
    elif 'datal' in stadatar.__dict__:
        data = stadatar.datal
    else:
        raise ValueError('Field \'datar\' or \'datal\' must be in the SACStation')
    dep_mod = DepModel(YAxisRange, velmod)
    # x_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    # x_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    Tpds = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    for i in range(stadatar.ev_num):
        # x_s[i] = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vs) ** -2)) - 1))
        # x_p[i] = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vp) ** -2)) - 1))
        Tpds[i] = np.cumsum((np.sqrt((dep_mod.R / dep_mod.vs) ** 2 - stadatar.rayp[i] ** 2) -
                             np.sqrt((dep_mod.R / dep_mod.vp) ** 2 - stadatar.rayp[i] ** 2)) * (dep_mod.dz / dep_mod.R))
    Tpds_ref = np.cumsum((np.sqrt((dep_mod.R / dep_mod.vs) ** 2 - raypref ** 2) -
                          np.sqrt((dep_mod.R / dep_mod.vp) ** 2 - raypref ** 2)) * (dep_mod.dz / dep_mod.R))

    Newdatar = np.zeros([stadatar.ev_num, stadatar.rflength])
    EndIndex = np.zeros(stadatar.ev_num)
#
    for i in range(stadatar.ev_num):
        Newaxis = np.array([])
        TempTpds = Tpds[i, :]
        StopIndex = np.where(np.imag(TempTpds) == 1)[0]
        if StopIndex.size == 0:
            StopIndex = dep_mod.depths.shape[0]
        EndIndex[i] = StopIndex - 1
        Newaxis = np.append(Newaxis, np.append(np.arange(-shift, 0, sampling), 0))
        for j in np.arange(int(shift / sampling + 1), stadatar.rflength):
            Refaxis = j * sampling - shift
            index = np.where(Refaxis <= Tpds[i, 0:StopIndex])[0]
            if index.size == 0:
                break
            Ratio = (Tpds_ref[index[0]] - Tpds_ref[index[0] - 1]) / (Tpds[i, index[0]] - Tpds[i, index[0] - 1])
            Newaxis = np.append(Newaxis, Tpds_ref[index[0] - 1] + (Refaxis - Tpds[i, index[0] - 1]) * Ratio)
        endidx = Newaxis.shape[0]
        x_new = np.arange(0, stadatar.rflength) * sampling - shift
        Tempdata = interp1d(Newaxis, data[i, 0:endidx], bounds_error=False)(x_new)
        endIndice = np.where(np.isnan(Tempdata))[0]
        if endIndice.size == 0:
            New_data = Tempdata
        else:
            New_data = np.append(Tempdata[1:endIndice[0]], data[i, endidx+1:])
        if New_data.shape[0] < stadatar.rflength:
            Newdatar[i] = np.append(New_data, np.zeros(stadatar.rflength - New_data.shape[0]))
        else:
            Newdatar[i] = New_data[0: stadatar.rflength]
    return Newdatar, EndIndex


def psrf2depth(stadatar, YAxisRange, sampling, shift, velmod='iasp91', srayp=None):
    """
    :param stadatar:
    :param YAxisRange:
    :param sampling:
    :param shift:
    :param velmod:
    :return:
    """
    if exists(velmod):
        try:
            dep_mod = DepModel(YAxisRange, velmod)
        except:
            dep_mod = DepModel(YAxisRange, 'iasp91')
            try:
                velmod_3d = np.load(velmod)
                dep_mod.vp, dep_mod.vs = interp_depth_model(velmod_3d, stadatar.stla, stadatar.stlo, YAxisRange)
            except Exception as e:
                raise FileNotFoundError('Cannot load 1D or 3D velocity model of \'{}\''.format(velmod))
    else:
        try:
            dep_mod = DepModel(YAxisRange, velmod)
        except:
            raise ValueError('Cannot recognize the velocity model of \'{}\''.format(velmod))

    x_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    x_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    Tpds = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    if srayp is None:
        for i in range(stadatar.ev_num):
            x_s[i] = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vs) ** -2)) - 1))
            x_p[i] = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vp) ** -2)) - 1))
            Tpds[i] = np.cumsum((np.sqrt((dep_mod.R / dep_mod.vs) ** 2 - stadatar.rayp[i] ** 2) -
                                 np.sqrt((dep_mod.R / dep_mod.vp) ** 2 - stadatar.rayp[i] ** 2)) * (dep_mod.dz / dep_mod.R))
    elif isinstance(srayp, str) or isinstance(srayp, np.lib.npyio.NpzFile):
        if isinstance(srayp, str):
            if not exists(srayp):
                raise FileNotFoundError('Ps rayp lib file was not found')
            else:
                rayp_lib = np.load(srayp)
        else:
            rayp_lib = srayp
        for i in range(stadatar.ev_num):
            rayp = get_psrayp(rayp_lib, stadatar.dis[i], stadatar.evdp[i], dep_mod.depths)
            rayp = skm2srad(sdeg2skm(rayp))
            x_s[i] = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (rayp ** 2. * (dep_mod.R / dep_mod.vs) ** -2)) - 1))
            x_p[i] = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vp) ** -2)) - 1))
            Tpds[i] = np.cumsum((np.sqrt((dep_mod.R / dep_mod.vs) ** 2 - rayp ** 2) -
                                 np.sqrt((dep_mod.R / dep_mod.vp) ** 2 - stadatar.rayp[i] ** 2)) * (dep_mod.dz / dep_mod.R))
    else:
        raise TypeError('srayp should be path to Ps rayp lib')

    time_axis = np.arange(0, stadatar.rflength) * sampling - shift
    PS_RFdepth = np.zeros([stadatar.ev_num, dep_mod.depths.shape[0]])
    EndIndex = np.zeros(stadatar.ev_num)
    for i in range(stadatar.ev_num):
        TempTpds = Tpds[i, :]
        StopIndex = np.where(np.imag(TempTpds) == 1)[0]
        if StopIndex.size == 0:
            EndIndex[i] = dep_mod.depths.shape[0]
            DepthAxis = interp1d(TempTpds, dep_mod.depths, bounds_error=False)(time_axis)
        else:
            EndIndex[i] = StopIndex[0] - 1
            DepthAxis = interp1d(TempTpds[0:StopIndex], dep_mod.depths[0: StopIndex], bounds_error=False)(time_axis)

        PS_RFTempAmps = stadatar.datar[i]
        ValueIndices = np.where(np.logical_not(np.isnan(DepthAxis)))[0]

        if ValueIndices.size == 0:
            continue
        elif np.max(ValueIndices) > PS_RFTempAmps.shape[0]:
            continue
        else:
            PS_RFAmps = interp1d(DepthAxis[ValueIndices], PS_RFTempAmps[ValueIndices], bounds_error=False)(YAxisRange)
            PS_RFdepth[i] = PS_RFAmps / np.nanmax(PS_RFAmps)
    return PS_RFdepth, EndIndex, x_s, x_p


def psrf_1D_raytracing(stadatar, YAxisRange, velmod='iasp91', srayp=None):
    dep_mod = DepModel(YAxisRange, velmod)

    # x_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    raylength_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplat_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplon_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    # x_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    raylength_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplat_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplon_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    Tpds = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    if srayp is None:
        for i in range(stadatar.ev_num):
            x_s = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vs) ** -2)) - 1))
            raylength_s[i] = (dep_mod.dz * dep_mod.R) / (np.sqrt(((dep_mod.R / dep_mod.vs) ** 2) - (stadatar.rayp[i] ** 2)) * dep_mod.vs)
            x_p = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vp) ** -2)) - 1))
            raylength_p[i] = (dep_mod.dz * dep_mod.R) / (np.sqrt(((dep_mod.R / dep_mod.vp) ** 2) - (stadatar.rayp[i] ** 2)) * dep_mod.vp)
            Tpds[i] = np.cumsum((np.sqrt((dep_mod.R / dep_mod.vs) ** 2 - stadatar.rayp[i] ** 2) -
                                 np.sqrt((dep_mod.R / dep_mod.vp) ** 2 - stadatar.rayp[i] ** 2)) * (dep_mod.dz / dep_mod.R))
            pplat_s[i], pplon_s[i] = latlon_from(stadatar.stla, stadatar.stlo, stadatar.bazi[i], rad2deg(x_s))
            pplat_p[i], pplon_p[i] = latlon_from(stadatar.stla, stadatar.stlo, stadatar.bazi[i], rad2deg(x_p))
    elif isinstance(srayp, str) or isinstance(srayp, np.lib.npyio.NpzFile):
        if isinstance(srayp, str):
            if not exists(srayp):
                raise FileNotFoundError('Ps rayp lib file not found')
            else:
                rayp_lib = np.load(srayp)
        else:
            rayp_lib = srayp
        for i in range(stadatar.ev_num):
            rayp = get_psrayp(rayp_lib, stadatar.dis[i], stadatar.evdp[i], dep_mod.depths)
            rayp = skm2srad(sdeg2skm(rayp))
            x_s = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (rayp ** 2. * (dep_mod.R / dep_mod.vs) ** -2)) - 1))
            raylength_s[i] = (dep_mod.dz * dep_mod.R) / (np.sqrt(((dep_mod.R / dep_mod.vs) ** 2) - (rayp ** 2)) * dep_mod.vs)
            x_p = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (stadatar.rayp[i] ** 2. * (dep_mod.R / dep_mod.vp) ** -2)) - 1))
            raylength_p[i] = (dep_mod.dz * dep_mod.R) / (np.sqrt(((dep_mod.R / dep_mod.vp) ** 2) - (stadatar.rayp[i] ** 2)) * dep_mod.vp)
            Tpds[i] = np.cumsum((np.sqrt((dep_mod.R / dep_mod.vs) ** 2 - rayp ** 2) -
                                 np.sqrt((dep_mod.R / dep_mod.vp) ** 2 - stadatar.rayp[i] ** 2)) * (dep_mod.dz / dep_mod.R))
            x_s = _imag2nan(x_s)
            x_p = _imag2nan(x_p)
            pplat_s[i], pplon_s[i] = latlon_from(stadatar.stla, stadatar.stlo, stadatar.bazi[i], rad2deg(x_s))
            pplat_p[i], pplon_p[i] = latlon_from(stadatar.stla, stadatar.stlo, stadatar.bazi[i], rad2deg(x_p))
    else:
        raise TypeError('srayp should be path to Ps rayp lib')
    return pplat_s, pplon_s, pplat_p, pplon_p, raylength_s, raylength_p, Tpds


def psrf_3D_raytracing(stadatar, YAxisRange, mod3d, srayp=None):
    """
Back ray trace the S wavs with a assumed ray parameter of P.

Parameters
--------------
stla: float
    The latitude of the station
stlo: float
    The longitude of the station
stadatar: object SACStation
    The data class including PRFs and more parameters
YAxisRange: array_like
    The depth array with the same intervals
mod3d: 'Mod3DPerturbation' object
    The 3D velocity model with fields of ``dep``, ``lat``,
    ``lon``, ``vp`` and ``vs``.
    """
    R = 6371 - YAxisRange
    ddepth = np.mean(np.diff(YAxisRange))
    pplat_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplon_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplat_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    pplon_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    x_s = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    x_p = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    Tpds = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    rayps = srad2skm(stadatar.rayp)

    if isinstance(srayp, str) or isinstance(srayp, np.lib.npyio.NpzFile):
        if isinstance(srayp, str):
            if not exists(srayp):
                raise FileNotFoundError('Ps rayp lib file not found')
            else:
                rayp_lib = np.load(srayp)
        else:
            rayp_lib = srayp
    elif srayp is None:
        pass
    else:
        raise TypeError('srayp should be path to Ps rayp lib')

    for i in range(stadatar.ev_num):
        if srayp is None:
            srayps = stadatar.rayp[i]
        else:
            srayps = get_psrayp(rayp_lib, stadatar.dis[i],
                                stadatar.evdp[i], YAxisRange)
            srayps = skm2srad(sdeg2skm(srayps))
        pplat_s[i][0] = pplat_p[i][0] = stadatar.stla
        pplon_s[i][0] = pplon_p[i][0] = stadatar.stlo
        x_s[i][0] = 0
        x_p[i][0] = 0
        vs = np.zeros_like(YAxisRange)
        vp = np.zeros_like(YAxisRange)
        for j, dep in enumerate(YAxisRange[:-1]):
            vs[j] = interpn((mod3d.model['dep'], mod3d.model['lat'], mod3d.model['lon']),
                            mod3d.model['vs'], (dep, pplat_s[i, j], pplon_s[i, j]),
                            bounds_error=False, fill_value=None)
            vp[j] = interpn((mod3d.model['dep'], mod3d.model['lat'], mod3d.model['lon']),
                            mod3d.model['vp'], (dep, pplat_p[i, j], pplon_p[i, j]),
                            bounds_error=False, fill_value=None)
            x_s[i, j+1] = ddepth*tand(asind(vs[j]*rayps[i])) + x_s[i, j]
            x_p[i, j+1] = ddepth*tand(asind(vp[j]*rayps[i])) + x_p[i, j]
            pplat_s[i, j+1], pplon_s[i, j+1] = latlon_from(stadatar.stla,
                                                           stadatar.stlo,
                                                           stadatar.bazi[i],
                                                           km2deg(x_s[i, j+1]))
            pplat_p[i, j+1], pplon_p[i, j+1] = latlon_from(stadatar.stla,
                                                           stadatar.stlo,
                                                           stadatar.bazi[i],
                                                           km2deg(x_p[i, j+1]))
        Tpds[i] = np.cumsum((np.sqrt((R / vs) ** 2 - srayps ** 2) -
                            np.sqrt((R / vp) ** 2 - stadatar.rayp[i] ** 2))
                            * (ddepth / R))
    return pplat_s, pplon_s, pplat_p, pplon_p, Tpds


def interp_depth_model(model, lat, lon, new_dep):
    #  model = np.load(modpath)
    points = [[depth, lat, lon] for depth in new_dep]
    vp = interpn((model['dep'], model['lat'], model['lon']), model['vp'], points, bounds_error=False, fill_value=None)
    vs = interpn((model['dep'], model['lat'], model['lon']), model['vs'], points, bounds_error=False, fill_value=None)
    return vp, vs


class Mod3DPerturbation:
    def __init__(self, modpath, YAxisRange, velmod='iasp91'):
        dep_mod = DepModel(YAxisRange, velmod=velmod)
        self.model = np.load(modpath)
        new1dvp = interp1d(dep_mod.depthsraw, dep_mod.vpraw)(self.model['dep'])
        new1dvs = interp1d(dep_mod.depthsraw, dep_mod.vsraw)(self.model['dep'])
        new1dvp, _, _ = np.meshgrid(new1dvp, self.model['lat'], self.model['lon'], indexing='ij')
        new1dvs, _, _ = np.meshgrid(new1dvs, self.model['lat'], self.model['lon'], indexing='ij')
        self.dvp = (self.model['vp'] - new1dvp) / new1dvp
        self.dvs = (self.model['vs'] - new1dvs) / new1dvs
        self.cvp = dep_mod.vp
        self.cvs = dep_mod.vs

    def interpdvp(self, points):
        dvp = interpn((self.model['dep'], self.model['lat'], self.model['lon']), self.dvp, points,
                      bounds_error=False, fill_value=None)
        return dvp

    def interpdvs(self, points):
        dvs = interpn((self.model['dep'], self.model['lat'], self.model['lon']), self.dvs, points,
                      bounds_error=False, fill_value=None)
        return dvs


def psrf_3D_migration(pplat_s, pplon_s, pplat_p, pplon_p, raylength_s, raylength_p, Tpds, YAxisRange, mod3d):
    ev_num, _ = raylength_p.shape
    timecorrections = np.zeros_like(raylength_p)
    for i in range(ev_num):
        points = np.array([YAxisRange, pplat_p[i], pplon_p[i]]).T
        dvp = mod3d.interpdvp(points)
        points = np.array([YAxisRange, pplat_s[i], pplon_s[i]]).T
        dvs = mod3d.interpdvs(points)
        dlp = raylength_p[i]
        dls = raylength_s[i]
        tmpds = (dls / (mod3d.cvs * (1 + dvs)) - dls / mod3d.cvs) - (dlp / (mod3d.cvp * (1 + dvp)) - dlp / mod3d.cvp)
        tmpds[np.isnan(tmpds)] = 0
        timecorrections[i] = np.cumsum(tmpds)
    return Tpds + timecorrections


def time2depth(stadatar, YAxisRange, Tpds):
    time_axis = np.arange(0, stadatar.RFlength) * stadatar.sampling - stadatar.shift
    PS_RFdepth = np.zeros([stadatar.ev_num, YAxisRange.shape[0]])
    EndIndex = np.zeros(stadatar.ev_num)
    for i in range(stadatar.ev_num):
        TempTpds = Tpds[i, :]
        StopIndex = np.where(np.imag(TempTpds) == 1)[0]
        if StopIndex.size == 0:
            EndIndex[i] = YAxisRange.shape[0]
            DepthAxis = interp1d(TempTpds, YAxisRange, bounds_error=False)(time_axis)
        else:
            EndIndex[i] = StopIndex[0] - 1
            DepthAxis = interp1d(TempTpds[0:StopIndex], YAxisRange[0: StopIndex], bounds_error=False)(time_axis)

        PS_RFTempAmps = stadatar.datar[i]
        ValueIndices = np.where(np.logical_not(np.isnan(DepthAxis)))[0]

        if ValueIndices.size == 0:
            continue
        elif np.max(ValueIndices) > PS_RFTempAmps.shape[0]:
            continue
        else:
            PS_RFAmps = interp1d(DepthAxis[ValueIndices], PS_RFTempAmps[ValueIndices], bounds_error=False)(YAxisRange)
            PS_RFdepth[i] = PS_RFAmps / np.nanmax(PS_RFAmps)
    return PS_RFdepth, EndIndex


if __name__ == '__main__':
    rfsta = SACStation('/Users/xumijian/Codes/seispy-example/ex-ccp/RFresult/ZX.212/ZX.212finallist.dat')
    rfsta.jointani(2, 7, weight=[0.9, 0.1, 0.0])
    rfsta.ani.plot_polar()




from datetime import datetime
from os.path import dirname, join
import urllib.request as rq
import re
import argparse
import sys
import os


def convertinfo(info):
    if info[5] == '60.0':
        sec = 59
    else:
        sec = float(info[5])
    return int(info[0]), int(info[1]), int(info[2]), int(info[3]), int(info[4]), sec, float(info[6]), float(info[7]), \
           float(info[8]), float(info[9])


def ndkparse(ndk_str):
    find_re = re.compile(
        r'[A-Z]+\s(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):(\d+.\d)\s+(.+?)\s+(.+?)\s+(.+?)\s+.+?\s+(.+?)\s+.+?\n', re.DOTALL)
    ndk_lst = [convertinfo(info) for info in find_re.findall(ndk_str)]
    return ndk_lst


def fetch_cata(inlog=join(dirname(__file__), 'data', 'EventCMT.dat'), outlog=''):
    url = 'http://www.ldeo.columbia.edu/~gcmt/projects/CMT/catalog/NEW_QUICK/qcmt.ndk'
    try:
        print('Connecting to http://www.ldeo.columbia.edu/.../qcmt.ndk')
        response = rq.urlopen(url)
    except Exception as e:
        raise TimeoutError('Could not connect to http://www.ldeo.columbia.edu/~gcmt/projects/CMT/'
                           'catalog/NEW_QUICK/qcmt.ndk\n{}'.format(e))

    html = response.read().decode('utf-8')
    with open(inlog, 'r') as fid_old:
        all_old_log = fid_old.readlines()
    old_log = all_old_log[-1]
    old_time_end = datetime(int(old_log.split()[0]), int(old_log.split()[1]), int(old_log.split()[2]),
                            int(old_log.split()[4]), int(old_log.split()[5]), int(old_log.split()[6]))

    if outlog == '':
        fid_new = open(inlog, 'w+')
        fid_new.writelines(all_old_log)
    else:
        fid_new = open(outlog, 'a+')

    print('Writing event info')
    for year, mon, day, hour, min, sec, lat, lon, dep, mw in ndkparse(html):
        evt_time = datetime(year, mon, day, hour, min, int(sec))
        if old_time_end < evt_time:
            fid_new.write('%d %d %d %s %d %d %d %6.2f %6.2f %s %s\n' % (
            year, mon, day, evt_time.strftime('%j'), hour, min, int(sec), lat, lon, dep, mw))
    fid_new.close()


def main():
    parser = argparse.ArgumentParser(description="Update CMT Catalog")
    parser.add_argument('-i', help='Input Catalog', dest='inlog', metavar='input_catalog',
                        type=str, default=join(dirname(__file__), 'data', 'EventCMT.dat'))
    parser.add_argument('-o', help='Onput Catalog', dest='outlog', metavar='output_catalog', 
                        type=str, default=join(dirname(__file__), 'data', 'EventCMT.dat'))
    # parser.add_argument('-u', help='url of ndk', metavar='ndl_url', type=str)
    arg = parser.parse_args()
    fetch_cata(inlog=arg.inlog, outlog=arg.outlog)


def ndk2dat():
    parser = argparse.ArgumentParser(description="Convert ndk file to dat catalog")
    parser.add_argument('-i', help='Input Catalog', dest='inlog', metavar='input_catalog',
                        type=str)
    parser.add_argument('-o', help='Onput Catalog', dest='outlog', metavar='output_catalog', 
                        type=str, default='EventCMT.dat')
    arg = parser.parse_args()
    with open(arg.inlog) as f:
        content = f.read()
    try:
        log_lst = ndkparse(content)
    except Exception as e:
        raise TypeError('Error type for ndk file\n{}'.format(e))
    with open(arg.outlog, 'w+') as f:
        for year, mon, day, hour, min, sec, lat, lon, dep, mw in log_lst:
            evt_time = datetime(year, mon, day, hour, min, int(sec))
            f.write('%d %d %d %s %d %d %d %6.2f %6.2f %s %s\n' % (
            year, mon, day, evt_time.strftime('%j'), hour, min, int(sec), lat, lon, dep, mw))


if __name__ == '__main__':
    ndk2dat()



from math import e
import sys
import os
import argparse
# matplotlib.use("Qt5Agg")
from PyQt5.QtGui import QIcon, QKeySequence
from PyQt5.QtWidgets import QApplication, QMainWindow, QVBoxLayout, \
                            QSizePolicy, QWidget, QDesktopWidget, \
                            QPushButton, QHBoxLayout, QFileDialog, \
                            QAction, QShortcut
from os.path import exists, dirname, join
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
from seispy.pickfigure import RFFigure
from seispy.rpickfigure import RPickFigure
import glob


class MyMplCanvas(FigureCanvas):
    def __init__(self, parent=None, rfpath='', only_r=False, width=21, height=11, dpi=100, xlim=[-2, 30]):

        plt.rcParams['axes.unicode_minus'] = False 

        if only_r:
            self.rffig = RPickFigure(rfpath, width=width, height=height, dpi=dpi, xlim=xlim)
            self.rffig.init_canvas()
        else:
            self.rffig = RFFigure(rfpath, width=width, height=height, dpi=dpi, xlim=xlim)
            self.rffig.init_canvas()

        FigureCanvas.__init__(self, self.rffig.fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Expanding,
                                   QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)


class MatplotlibWidget(QMainWindow):
    def __init__(self, rfpath, only_r=False, xlim=[-2, 30], parent=None):
        super(MatplotlibWidget, self).__init__(parent)
        self.only_r = only_r
        self.initUi(rfpath, only_r, xlim)

    def initUi(self, rfpath, only_r, xlim):
        self.layout = QVBoxLayout()
        self.add_btn()
        self.mpl = MyMplCanvas(self, rfpath=rfpath, only_r=only_r, width=21, height=11, dpi=100, xlim=xlim)
        self.layout.addWidget(self.mpl, 2)
        self.mpl.mpl_connect('button_press_event', self.on_click)

        main_frame = QWidget()
        self.setCentralWidget(main_frame)
        main_frame.setLayout(self.layout)

        saveAction = QAction('&Save', self)        
        saveAction.setShortcut('Ctrl+S')
        saveAction.setStatusTip('Save this figure')
        saveAction.triggered.connect(self.plot_save)

        menubar = self.menuBar()
        fileMenu = menubar.addMenu('&File')
        fileMenu.addAction(saveAction)

        self._set_geom_center()
        self._define_global_shortcuts()
        self.setWindowTitle('PickRF')
        self.setWindowIcon(QIcon(join(dirname(__file__), 'data', 'seispy.png')))

    def on_click(self, event):
        self.mpl.rffig.onclick(event)
        self.mpl.draw()

    def previous_connect(self):
        self.mpl.rffig.butprevious()
        self.mpl.draw()

    def next_connect(self):
        self.mpl.rffig.butnext()
        self.mpl.draw()

    def enlarge(self):
        self.mpl.rffig.enlarge()
        self.mpl.draw()

    def reduce(self):
        self.mpl.rffig.reduce()
        self.mpl.draw()

    def finish(self):
        self.mpl.rffig.finish()
        QApplication.quit()

    def plot_ui(self):
        self.mpl.rffig.plot()

    def plot_save(self):
        if self.only_r:
            default_name = 'R_bazorder'
        else:
            default_name = 'RT_bazorder'
        fileName_choose, filetype = QFileDialog.getSaveFileName(self,
                                    "Save the figure",
                                    os.path.join(os.getcwd(), self.mpl.rffig.staname + default_name), 
                                    "PDF Files (*.pdf);;Images (*.png);;All Files (*)")

        if fileName_choose == "":
            return
        if not hasattr(self.mpl.rffig, 'plotfig'):
            self.mpl.rffig.plot()
        try:
            self.mpl.rffig.plotfig.savefig(fileName_choose, dpi=500, bbox_inches='tight')
            self.mpl.rffig.log.RFlog.info('Figure saved to {}'.format(fileName_choose))
        except Exception as e:
            self.mpl.rffig.log.RFlog.error('{}'.format(e))

    def _set_geom_center(self, height=1, width=1):
        screen_resolution = QDesktopWidget().screenGeometry()
        screen_height = screen_resolution.height()
        screen_width = screen_resolution.width()
        frame_height = int(screen_height * height)
        frame_width = int(screen_width * width)

        self.setGeometry(0, 0, frame_width, frame_height)
        self.move((screen_width / 2) - (self.frameSize().width() / 2),
                  (screen_height / 2) - (self.frameSize().height() / 2))

    def _define_global_shortcuts(self):
        self.key_c = QShortcut(QKeySequence('c'), self)
        self.key_c.activated.connect(self.next_connect)
        self.key_z = QShortcut(QKeySequence('z'), self)
        self.key_z.activated.connect(self.previous_connect)
        self.key_space = QShortcut(QKeySequence('Space'), self)
        self.key_space.activated.connect(self.plot_ui)

    def add_btn(self):
        pre_btn = QPushButton("Back (z)")
        pre_btn.clicked.connect(self.previous_connect)
        next_btn = QPushButton("Next (c)")
        next_btn.clicked.connect(self.next_connect)
        plot_btn = QPushButton("Preview (Space)")
        plot_btn.clicked.connect(self.plot_ui)
        finish_btn = QPushButton("Finish")
        finish_btn.clicked.connect(self.finish)
        btnbox = QHBoxLayout()
        btnbox.addStretch(1)
        btnbox.addWidget(pre_btn)
        btnbox.addWidget(next_btn)
        btnbox.addWidget(plot_btn)
        btnbox.addWidget(finish_btn)

        enlarge_btn = QPushButton("Amp enlarge")
        enlarge_btn.clicked.connect(self.enlarge)
        areduce_btn = QPushButton("Amp reduce")
        areduce_btn.clicked.connect(self.reduce)
        pathbox = QHBoxLayout()
        pathbox.addWidget(enlarge_btn)
        pathbox.addWidget(areduce_btn)

        ctrl_layout = QHBoxLayout()
        ctrl_layout.addLayout(pathbox)
        ctrl_layout.addLayout(btnbox)

        self.layout.addLayout(ctrl_layout)


def main():
    parser = argparse.ArgumentParser(description="User interface for picking PRFs")
    parser.add_argument('rf_path', type=str, help='Path to PRFs')
    parser.add_argument('-x', help="Set the x limits of the current axes, defaults to 30s for RT, 85s for R.",
                        dest='xlim', default=None, type=float, metavar='xmax')
    arg = parser.parse_args()
    rfpath = arg.rf_path
    if not exists(rfpath):
        raise FileNotFoundError('No such directory of {}'.format(rfpath))
    if len(glob.glob(join(rfpath, '*_T.sac'))) == 0:
        only_r = True
        xlim = 85
    else:
        only_r = False
        xlim = 30
    if arg.xlim is not None:
        xlim = arg.xlim
    app = QApplication(sys.argv)
    ui = MatplotlibWidget(rfpath, only_r=only_r, xlim=[-2, xlim])
    ui.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



import numpy as np


def smooth(x, half_len=5, window='flat'):
    """smooth the data using a window with requested size.

    This method is based on the convolution of a scaled window with the signal.
    The signal is prepared by introducing reflected copies of the signal
    (with the window size) in both ends so that transient parts are minimized
    in the begining and end part of the output signal.

    input:
        x: the input signal
        helf_len: the half dimension of the smoothing window; should be an odd integer
        window: the type of window from 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'
            flat window will produce a moving average smoothing.

    output:
        the smoothed signal

    example:

    t=linspace(-2,2,0.1)
    x=sin(t)+randn(len(t))*0.1
    y=smooth(x)

    see also:

    numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman, numpy.convolve
    scipy.signal.lfilter

    TODO: the window parameter could be the window itself if an array instead of a string
    NOTE: length(output) != length(input), to correct this: return y[(window_len/2-1):-(window_len/2)] instead of just y.
    """

    window_len = 2*half_len+1

    if x.ndim != 1:
        raise ValueError("smooth only accepts 1 dimension arrays.")

    if x.size < window_len:
        raise ValueError("Input vector needs to be bigger than window size.")

    if window_len < 3:
        return x

    if window not in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
        raise ValueError("Window is on of 'flat', 'hanning', 'hamming',"
                         "'bartlett', 'blackman'")
    s = np.r_[x[window_len-1:0:-1], x, x[-1:-window_len:-1]]
    if window == 'flat':
        w = np.ones(window_len, 'd')  # moving average
    else:
        w = eval('np.'+window+'(window_len)')

    y = np.convolve(w/w.sum(), s, mode='valid')
    return y[half_len:-half_len]


def whiten(data, Nfft, delta, f1, f2, f3, f4):
    """This function takes 1-dimensional *data* timeseries array,
    goes to frequency domain using fft, whitens the amplitude of the spectrum
    in frequency domain between *freqmin* and *freqmax*
    and returns the whitened fft.

    :type data: :class:`numpy.ndarray`
    :param data: Contains the 1D time series to whiten
    :type Nfft: int
    :param Nfft: The number of points to compute the FFT
    :type delta: float
    :param delta: The sampling frequency of the `data`
    :type freqmin: float
    :param freqmin: The lower frequency bound
    :type freqmax: float
    :param freqmax: The upper frequency bound
    :type plot: bool
    :param plot: Whether to show a raw plot of the action (default: False)

    :rtype: :class:`numpy.ndarray`
    :returns: The FFT of the input trace, whitened between the frequency bounds
"""
    dom = 1/delta/Nfft
    nt1 = int(f1/dom)
    nt2 = int(f2/dom)
    nt3 = int(f3/dom)
    nt4 = int(f4/dom)

    FFTRawSign = fftpack.fft(data, Nfft)

    FFTRawSign /= smooth(np.abs(FFTRawSign), half_len=20)
    # Left tapering:
    FFTRawSign[0:nt1] *= 0
    FFTRawSign[nt1:nt2] = np.cos(np.linspace(np.pi / 2., np.pi, nt2 - nt1)) ** 2 * np.exp(1j * np.angle(FFTRawSign[nt1:nt2]))
    #FFTRawSign[nt1:nt2] = np.cos(np.linspace(np.pi / 2., np.pi, nt2 - nt1)) ** 2 * FFTRawSign[nt1:nt2]
    # Pass band:
    FFTRawSign[nt2:nt3] = np.exp(1j * np.angle(FFTRawSign[nt2:nt3]))
    # Right tapering:
    FFTRawSign[nt3:nt4] = np.cos(np.linspace(0., np.pi / 2., nt4 - nt3)) ** 2 * np.exp(1j * np.angle(FFTRawSign[nt3:nt4]))
    #FFTRawSign[nt3:nt4] = np.cos(np.linspace(0., np.pi / 2., nt4 - nt3)) ** 2 * FFTRawSign[nt3:nt4]
    FFTRawSign[nt4:Nfft+1] *= 0
    
    # Hermitian symmetry (because the input is real)
    FFTRawSign[int(-Nfft/2+1):] = FFTRawSign[1:int(Nfft/2)].conjugate()[::-1]
    
    return FFTRawSign, dom




#!/usr/bin/env python
from setuptools import find_packages, setup
packages = find_packages()

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = "1.2.3"
setup(name='python-seispy',
      version=VERSION,
      author='Mijian Xu',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author_email='gomijianxu@gmail.com',
      license='GPLv3',
      packages=find_packages(),
      package_dir={'seispy': 'seispy'},
      package_data={'': ['data/*']},
      install_requires=[
                'netcdf4>=1.5.2',
                'pyerf>=1.0.1',
                'obspy>=1.2.0',
                'pandas>=1.0.0',
                'numpy>=1.19.0',
                'scipy>=1.1.0',
                'matplotlib>=3.2.0',
                'pyqt5>=5.12.0'],
      entry_points={'console_scripts': ['gen_rayp_lib=seispy.psrayp:gen_rayp_lib',
                                        'prf=seispy.rf:prf',
                                        'setpar=seispy.rf:setpar',
                                        'rf2depth=seispy.rf2depth_makedata:rf2depth',
                                        'plotrt=seispy.plotRT:main',
                                        'plotr=seispy.plotR:main',
                                        'updatecatalog=seispy.updatecatalog:main',
                                        'ndk2dat=seispy.updatecatalog:ndk2dat',
                                        'ccp_profile=seispy.scripts:ccp_profile',
                                        'hk=seispy.hk:hk',
                                        'pickrf=seispy.pickui:main',
                                        'rfani=seispy.scripts:rfani',
                                        'ccp3d=seispy.scripts:ccp3d',
                                        'get_pierce_points=seispy.scripts:get_pierce_points']},
      #  include_package_data=True,
      zip_safe=False,
      classifiers=['Programming Language :: Python',
                   'Programming Language :: Python :: 3.7',
                   'Programming Language :: Python :: 3.8',
                   'Programming Language :: Python :: 3.9']
      )



import os
import numpy as np
from scipy.interpolate import interpn
import subprocess
import argparse
import sys


class PsRayp(object):
    def __init__(self, dis, dep, laymin=0, laymax=800):
        self.dis = dis
        self.dep = dep
        self.layers = np.arange(laymin, laymax)
        self.real_layers = np.array([])
        self.fake_layers = np.array([])
        self.real_idx = np.array([])
        self.fake_idx = np.array([])
        self.rayp = np.zeros((len(dis), len(dep), len(self.layers)))

    def make_phase_list(self):
        self.real_idx = np.where(self.layers >= 11)[0]
        self.fake_idx = np.where(self.layers < 11)[0]
        if self.real_idx.size == 0:
            raise ValueError('Max layer must greater than 8 km')
        self.real_layers = self.layers[self.real_idx]
        self.fake_layers = self.layers[self.fake_idx]
        with open('/tmp/tmp_phlst.txt', 'w+') as f:
            for lay in self.real_layers:
                f.write('P{}s\n'.format(lay))

    def taup_rayp(self, this_dis=50, this_dep=10, taup='taup_time'):
        """
        :param taup: Path to taup_time
        :return:
        """
        if not os.path.exists('/tmp/tmp_phlst.txt'):
            raise FileNotFoundError('Please excute \'make_phase_list\' first')
        cmd = '{} -h {} -pf {} -deg {} --rayp'.format(taup, this_dep, '/tmp/tmp_phlst.txt', this_dis)
        s = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        # s.communicate(cmd.encode())
        rayp = np.array(s.stdout.read().decode().strip().split())
        out_rayp = np.zeros([2, self.layers.shape[0]])
        for lay in self.fake_idx:
            out_rayp[0, lay] = self.layers[lay]
            out_rayp[1, lay] = rayp[0]
        for lay in self.real_idx:
            out_rayp[0, lay] = self.layers[lay]
            out_rayp[1, lay] = rayp[lay-11]
        return out_rayp

    def get_rayp(self):
        for i in range(self.dis.shape[0]):
            print('{}'.format(self.dis[i]))
            for j in range(self.dep.shape[0]):
                self.rayp[i, j, :] = self.taup_rayp(this_dis=self.dis[i], this_dep=self.dep[j])[1]

    def save(self, path='Ps_rayp'):
        np.savez(path, dis=self.dis, dep=self.dep, layers=self.layers, rayp=self.rayp)


def gen_rayp_lib():
    parser = argparse.ArgumentParser(description="Gen a ray parameter lib for Pds phases")
    parser.add_argument('-d', help='Distance range in degree ',
                        metavar='min_dis/max_dis/interval', required=True, dest='dis_str', type=str)
    parser.add_argument('-e', help='Event depth range in km',
                        metavar='min_dep/max_dep/interval', required=True, dest='dep_str', type=str)
    parser.add_argument('-l', help='layers range as in km, defaults to 0/800',
                        dest='lay_str', metavar='min_layer/man_layer', type=str, default='0/800')
    parser.add_argument('-o', help='Out path to Pds ray parameter lib',
                        metavar='outpath', type=str, default='./Ps_rayp', dest='out_path')

    arg = parser.parse_args()
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    dis_lst = [float(dd) for dd in arg.dis_str.split('/')]
    dis = np.arange(dis_lst[0], dis_lst[1], dis_lst[2])
    dep_lst = [float(dd) for dd in arg.dep_str.split('/')]
    dep = np.arange(dep_lst[0], dep_lst[1], dep_lst[2])
    laymin = int(arg.lay_str.split('/')[0])
    laymax = int(arg.lay_str.split('/')[1])

    pr = PsRayp(dis, dep, laymin=laymin, laymax=laymax)
    pr.make_phase_list()
    pr.get_rayp()
    pr.save(path=arg.out_path)


def get_psrayp(rayp_lib, dis, dep, layers):
    # x_layers = np.zeros([len(layers), 3])
    # for i in range(len(layers)):
    #     x_layers[i] = np.array([dis, dep, layers[i]])
    x_layers = np.array([[dis, dep, lay]for lay in layers])
    return interpn((rayp_lib['dis'], rayp_lib['dep'], rayp_lib['layers']), rayp_lib['rayp'], x_layers,
                   bounds_error=False, fill_value=None)


if __name__ == '__main__':
    layers = np.arange(11, 300)
    dep = np.arange(0, 600, 2)
    dis = np.arange(30, 90)
    rayp_path = '/Users/xumj/Researches/Ps_rayp.npz'
    rayp_lib = np.load(rayp_path)
    print(get_psrayp(rayp_lib, 50, 10, np.arange(0,100)))



import numpy as np
from seispy.geo import km2deg, deg2km, latlon_from, geoproject, sind, rad2deg, skm2srad
from seispy.setuplog import setuplog
from seispy.distaz import distaz
from seispy.rfcorrect import DepModel
from seispy.ccppara import ccppara, CCPPara
from seispy.bootstrap import ci
from seispy.ccp3d import boot_bin_stack
from seispy.utils import check_stack_val, read_rfdep
from os.path import exists, dirname, basename, join


def line_proj(lat1, lon1, lat2, lon2):
    daz = distaz(lat1, lon1, lat2, lon2)
    az1_begin = (daz.baz - 90) % 360
    az2_begin = (daz.baz + 90) % 360
    az1_end = (daz.az - 90) % 360
    az2_end = (daz.az + 90) % 360
    return az1_begin, az2_begin, az1_end, az2_end, daz


def fix_filename(filename, typ='dat'):
    dname = dirname(filename)
    if not exists(dname) and dname != '':
        raise FileExistsError('internal error')
    bname = basename(filename)
    sp_name = bname.split('.')
    if len(sp_name) == 1:
        return filename + '.' + typ
    if sp_name[-1] != typ:
        nname = '.'.join(sp_name[0:-1]) + '.' + typ
        return join(dirname(filename), nname)
    else:
        return filename


def bin_shape(cpara):
    if cpara.bin_radius is None:
        depmod = DepModel(cpara.stack_range)
        fzone = km2deg(np.sqrt(0.5*cpara.domperiod*depmod.vs*cpara.stack_range))
    else:
        fzone = np.ones_like(cpara.stack_range) * km2deg(cpara.bin_radius)
    return fzone


def init_profile(lat1, lon1, lat2, lon2, val):
    """ Initial bins along a profile with given position of two points.

    :param lat1: The latitude of the start point
    :type lat1: float
    :param lon1: The lontitude of the start point
    :type lon1: float
    :param lat2: The latitude of the end point
    :type lat2: float
    :param lon2: The lontitude of the end point
    :type lon2: float
    :param val: The interval between two points in km
    :type val: float
    :return: The location of bins (bin_loca), and length between each bin and the start point (profile_range)

    The bin_loca is positions of bins with a numpy.array with two column. The profile_range is distance between bin center and the start point with an 1D numpy.array.
    :rtype: (numpy.array, numpy.array)
    """
    azi = distaz(lat1, lon1, lat2, lon2).baz
    dis = distaz(lat1, lon1, lat2, lon2).delta
    profile_range = np.arange(0, deg2km(dis), val)
    lat_loca, lon_loca = latlon_from(lat1, lon1, azi, km2deg(profile_range))
    bin_loca = np.zeros([lat_loca.shape[0], 2])
    bin_loca = np.vstack((lat_loca, lon_loca)).T
    return bin_loca, profile_range


class CCPProfile():
    def __init__(self, cfg_file=None, log=None):
        if log is None:
            self.logger = setuplog()
        else:
            self.logger = log
        if cfg_file is None:
            self.cpara = CCPPara()
        elif isinstance(cfg_file, str):
            self.load_para(cfg_file)
        else:
            raise ValueError('cfg_file must be str format.')
        self.stack_data = []

    def load_para(self, cfg_file):
        try:
            self.cpara = ccppara(cfg_file)
        except Exception as e:
            self.logger.CCPlog.error('Cannot open configure file {}'.format(cfg_file))
            raise FileNotFoundError('{}'.format(e))
        try:
            self.stack_mul = check_stack_val(self.cpara.stack_val, self.cpara.dep_val)
        except Exception as e:
            self.logger.CCPlog.error('{}'.format(e))
            raise ValueError('{}'.format(e))

    def read_rfdep(self):
        self.logger.CCPlog.info('Loading RFdepth data from {}'.format(self.cpara.depthdat))
        try:
            self.rfdep = read_rfdep(self.cpara.depthdat)
        except FileNotFoundError as e:
            self.logger.CCPlog.error('{}'.format(e))
            raise FileNotFoundError('Cannot open file of {}'.format(self.cpara.depthdat))
    
    def initial_profile(self):
        self.read_rfdep()
        self.bin_loca, self.profile_range = init_profile(*self.cpara.line, self.cpara.slide_val)
        self.fzone = bin_shape(self.cpara)
        self._get_sta()
        self._select_sta()
    
    def _get_sta(self):
        self.station = [sta['station'] for sta in self.rfdep]
        self.stalst = np.array([[sta['stalat'], sta['stalon']] for sta in self.rfdep])

    def _select_sta(self):
        if exists(self.cpara.stack_sta_list):
            self.logger.CCPlog.info('Use stacking stations in {}'.format(self.cpara.stack_sta_list))
            staname = list(np.loadtxt(self.cpara.stack_sta_list, dtype=np.dtype('U10'), usecols=(0,), ndmin=1, unpack=True))
            self.idxs = []
            for sta in staname:
                try:
                    idx = self.station.index(sta)
                    self.idxs.append(idx)
                except ValueError:
                    self.logger.CCPlog.warning('{} does not in RFdepth structure'.format(sta))
                if self.cpara.shape == 'rect':
                    self._pierce_project(self.rfdep[idx])
        elif self.cpara.width is None and self.cpara.shape == 'circle':
            dep_mod = DepModel(self.cpara.stack_range)
            x_s = np.cumsum((dep_mod.dz / dep_mod.R) / np.sqrt((1. / (skm2srad(0.085) ** 2. * (dep_mod.R / dep_mod.vs) ** -2)) - 1))
            dis = self.fzone[-1] + rad2deg(x_s[-1]) + 0.3
            # self.idxs = self._proj_sta(dis)
            self.idxs = []
            for i, bin_info in enumerate(self.bin_loca):
                self.idxs.append(np.where(distaz(bin_info[0], bin_info[1], self.stalst[:, 0], self.stalst[:, 1]).delta <= dis)[0])
        elif self.cpara.width is not None and self.cpara.shape == 'rect':
            self.logger.CCPlog.info('Select stations within {} km perpendicular to the profile'.format(self.cpara.width))
            self.idxs = self._proj_sta()
            self._write_sta()  
        else:
            if self.cpara.width is not None:
                self.logger.CCPlog.error('Width of profile was set, the bin shape of {} is invalid'.format(self.cpara.shape))
                raise ValueError('Width of profile was set, the bin shape of {} is invalid'.format(self.cpara.shape))
            else:
                self.logger.CCPlog.error('Width of profile was not set, the bin shape of {} is invalid'.format(self.cpara.shape))
                raise ValueError('Width of profile was not set, the bin shape of {} is invalid'.format(self.cpara.shape))

    def _write_sta(self):
        with open(self.cpara.stack_sta_list, 'w') as f:
            for idx in self.idxs:
                f.write('{}\t{:.3f}\t{:.3f}\n'.format(self.station[idx], self.stalst[idx,0], self.stalst[idx, 1]))
                self._pierce_project(self.rfdep[idx])      

    def _proj_sta(self, width):
        az1_begin, az2_begin, az1_end, az2_end, daz = line_proj(*self.cpara.line)
        az_sta_begin = distaz(self.stalst[:, 0], self.stalst[:, 1], self.cpara.line[0], self.cpara.line[1]).az
        az_sta_end = distaz(self.stalst[:, 0], self.stalst[:, 1], self.cpara.line[2], self.cpara.line[3]).az
        if 0 <= daz.baz < 90 or 270 <= daz.baz < 360:
            tmp_idx_begin = np.where(((az1_begin < az_sta_begin)&(az_sta_begin < 360)) | ((0 < az_sta_begin)&(az_sta_begin < az2_begin)))[0]
        else:
            tmp_idx_begin = np.where((az1_begin < az_sta_begin)&(az_sta_begin < az2_begin))[0]
        if 90 <= daz.az < 270:
            tmp_idx_end = np.where(((az1_end < az_sta_end) & (az_sta_end < az2_end)))[0]
        else:
            tmp_idx_end = np.where(((az1_end < az_sta_end) & (az_sta_end < 360)) | ((0 < az_sta_end) & (az_sta_end < az2_end)))[0]
        sta_daz = distaz(self.stalst[:, 0], self.stalst[:, 1], self.cpara.line[0], self.cpara.line[1])
        sta_dis = sind(daz.baz-sta_daz.az) * sta_daz.degreesToKilometers()
        proj_idx = np.where(np.abs(sta_dis) < width)[0]
        tmp_idx = np.intersect1d(tmp_idx_begin, tmp_idx_end)
        final_idx = np.intersect1d(tmp_idx, proj_idx).astype(int)
        if not final_idx.any():
            self.logger.CCPlog.error('No stations within the profile belt with width of {}'.format(width))
            raise ValueError('Satisfied stations not found')
        return final_idx

    def _pierce_project(self, rfsta):
        rfsta['projlat'] = np.zeros_like(rfsta['piercelat'])
        rfsta['projlon'] = np.zeros_like(rfsta['piercelon'])
        for i, dep in enumerate(self.cpara.depth_axis):
            rfsta['projlat'][:, i], rfsta['projlon'][:, i] = geoproject(rfsta['piercelat'][:, i], rfsta['piercelon'][:, i], *self.cpara.line)

    def stack(self):
        if self.cpara.shape == 'rect': 
            field_lat = 'projlat'
            field_lon = 'projlon'
        elif self.cpara.shape == 'circle':
            field_lat = 'piercelat'
            field_lon = 'piercelon'
        else:
            pass
        for i, bin_info in enumerate(self.bin_loca):
            boot_stack = {}
            bin_mu = np.zeros(self.cpara.stack_range.size)
            bin_ci = np.zeros([self.cpara.stack_range.size, 2])
            bin_count = np.zeros(self.cpara.stack_range.size)
            self.logger.CCPlog.info('{}/{} bin from {:.2f} km at lat: {:.3f} lon: {:.3f}'.format(i + 1, self.bin_loca.shape[0], self.profile_range[i], bin_info[0], bin_info[1]))
            if self.cpara.width is None and self.cpara.shape == 'circle':
                idxs = self.idxs[i]
            else:
                idxs = self.idxs
            for j, dep in enumerate(self.cpara.stack_range):
                idx = int(j * self.stack_mul + self.cpara.stack_range[0]/self.cpara.dep_val)
                bin_dep_amp = np.array([])
                for k in idxs:
                    fall_idx = np.where(distaz(self.rfdep[k][field_lat][:, idx], self.rfdep[k][field_lon][:, idx],
                                        bin_info[0], bin_info[1]).delta < self.fzone[j])[0]
                    bin_dep_amp = np.append(bin_dep_amp, self.rfdep[k]['moveout_correct'][fall_idx, idx])
                bin_mu[j], bin_ci[j], bin_count[j] = boot_bin_stack(bin_dep_amp, n_samples=self.cpara.boot_samples)
            boot_stack['bin_lat'] = bin_info[0]
            boot_stack['bin_lon'] = bin_info[1]
            boot_stack['profile_dis'] = self.profile_range[i]
            boot_stack['mu'] = bin_mu
            boot_stack['ci'] = bin_ci
            boot_stack['count'] = bin_count
            self.stack_data.append(boot_stack)   

    def save_stack_data(self, format='npz'):
        """If format is \'npz\', saving stacked data and parameters to local as a npz file. To load the file, please use data = np.load(fname, allow_pickle=True).
        data['cpara'] is the parameters when CCP stacking.
        data['stack_data'] is the result of stacked data.

        If format is \'dat\' the stacked data will be save into a txt file with 8 columns, including bin_lat, bin_lon, profile_dis, depth, amp, ci_low, ci_high and count.
        where bin_lat and bin_lon represent the position of each bin; profile_dis represents the distance in km between each bin and the start point of the profile; depth represents depth of each bin; amp means the stacked amplitude; ci_low and ci_high mean confidence interval with bootstrap method; count represents stacking number of each bin.

        :param fname: file name of stacked data
        :type fname: str
        """
        fname = fix_filename(self.cpara.stackfile, format)
        self.logger.CCPlog.info('Saving stacked data to {}'.format(fname))
        if not isinstance(self.cpara.stackfile, str):
            self.logger.CCPlog.error('fname should be in \'str\'')
            raise ValueError('fname should be in \'str\'')
        if format == 'npz':
            np.savez(self.cpara.stackfile, cpara=self.cpara, stack_data=self.stack_data)
        elif format == 'dat':
            with open(fname, 'w') as f:
                for i, bin in enumerate(self.stack_data):
                    for j, dep in enumerate(self.cpara.stack_range):
                        if dep == self.cpara.stack_range[-1]:
                            f.write('{:.4f}\t{:.4f}\t{:.4f}\t{:.2f} nan nan nan nan\n'.format(bin['bin_lat'], bin['bin_lon'],
                                                                                              self.profile_range[i], dep))
                        else:
                            f.write('{:.4f}\t{:.4f}\t{:.4f}\t{:.2f}\t{:.4f}\t{:.4f}\t{:.4f}\t{:d}\n'.format(bin['bin_lat'], bin['bin_lon'],
                                                                                    self.profile_range[i],
                                                                                    dep, bin['mu'][j], bin['ci'][j, 0],
                                                                                    bin['ci'][j, 1], int(bin['count'][j])))


if __name__ == '__main__':
    bin_loca, _ = init_profile(27.5, 94, 36.5, 92, 5)
    print(bin_loca.shape)



import re
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from matplotlib.lines import Line2D
from seispy.rfcorrect import SACStation
from seispy.rf import CfgParser
import argparse
import numpy as np
from os.path import join
import sys


def init_figure():
    h = plt.figure(figsize=(8, 10))
    gs = GridSpec(1, 3)
    gs.update(wspace=0.25)
    axr = plt.subplot(gs[0, 0:-1])
    axr.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    axb = plt.subplot(gs[0, -1])
    axb.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
    return h, axr, axb


def read_process_data(rfpath):
    stadata = SACStation(rfpath, only_r=True)
    idx = np.argsort(stadata.bazi)
    stadata.event = stadata.event[idx]
    stadata.bazi = stadata.bazi[idx]
    stadata.datar = stadata.datar[idx]
    return stadata


def plot_waves(axr, axb, stadata, enf=12):
    bound = np.zeros(stadata.rflength)
    for i in range(stadata.ev_num):
        datar = stadata.datar[i] * enf + (i + 1)
        # axr.plot(time_axis, stadata.datar[i], linewidth=0.2, color='black')
        axr.fill_between(stadata.time_axis, datar, bound + i+1, where=datar > i+1, facecolor='red',
                         alpha=0.7)
        axr.fill_between(stadata.time_axis, datar, bound + i+1, where=datar < i+1, facecolor='blue',
                         alpha=0.7)
    axb.scatter(stadata.bazi, np.arange(stadata.ev_num) + 1, s=7)


def set_fig(axr,  axb, stadata, xmin=-2, xmax=80):
    y_range = np.arange(stadata.ev_num) + 1
    x_range = np.arange(0, xmax+2, 5)
    space = 2

    # set axr
    axr.set_xlim(xmin, xmax)
    axr.set_xticks(x_range)
    axr.set_xticklabels(x_range, fontsize=8)
    axr.set_ylim(0, stadata.ev_num + space)
    axr.set_yticks(y_range)
    axr.set_yticklabels(stadata.event, fontsize=5)
    axr.set_xlabel('Time after P (s)', fontsize=13)
    axr.set_ylabel('Event', fontsize=13)
    axr.add_line(Line2D([0, 0], axr.get_ylim(), color='black'))
    axr.set_title('R components ({})'.format(stadata.staname), fontsize=16)

    # set axb
    axb.set_xlim(0, 360)
    axb.set_xticks(np.linspace(0, 360, 7))
    axb.set_xticklabels(np.linspace(0, 360, 7, dtype='i'), fontsize=8)
    axb.set_ylim(0, stadata.ev_num + space)
    axb.set_yticks(y_range)
    axb.set_yticklabels(y_range, fontsize=5)
    axb.set_xlabel(r'Back-azimuth ($^\circ$)', fontsize=13)


def plotr(rfpath, outpath='./', xlim=[-2, 80], enf=6, format='pdf'):
    h, axr, axb = init_figure()
    stadata = read_process_data(rfpath)
    plot_waves(axr, axb, stadata, enf=enf)
    set_fig(axr, axb, stadata, xlim[0], xlim[1])
    h.savefig(join(outpath, stadata.staname + '_R_bazorder_{:.1f}.{}'.format(
              stadata.f0[0], format)), format=format, dpi=500)


def main():
    parser = argparse.ArgumentParser(description="Plot R&T receiver functions")
    parser.add_argument('rfpath', help='Path to PRFs with a \'finallist.dat\' in it', type=str)
    parser.add_argument('-w', help="Time window from t1 to t2, defaults to -2/80", type=str, default='-2/80', metavar='t1/t2')
    parser.add_argument('-e', help='Enlargement factor, defaults to 6', dest='enf', type=float, default=6, metavar='enf')
    parser.add_argument('-o', help='Output path without file name, defaults to current path', dest='output', default='./', type=str, metavar='outpath')
    parser.add_argument('-t', help='Specify figure format. f = \'.pdf\', g = \'.png\', defaults to \'g\'',
                    dest='format', default='g', type=str, metavar='f|g')
    arg = parser.parse_args()
    try:
        xlim = [float(v) for v in arg.w.split('/')]
    except:
        raise ValueError("Error format in time window")
    if arg.format not in ('f', 'g'):
        raise ValueError('Error: The format must be in \'f\' and \'g\'')
    elif arg.format == 'g':
        fmt = 'png'
    elif arg.format == 'f':
        fmt = 'pdf'
    plotr(arg.rfpath, arg.output, enf=arg.enf, xlim=xlim, format=fmt)



if __name__ == '__main__':
    pass